//---------------------------------------------------------------------
// Copyright (c) 2013 Galaxy Far East Corp, Inc. All rights reserved
// Galaxy Far East Corp
//---------------------------------------------------------------------
//              The Contrl and Status Registers with I2C Slave
//---------------------------------------------------------------------
// FILE NAME      : i2c_csr.sv
// DEPARTMENT     : TS1, GFEC TW
// AUTHOR         : Slin
// AUTHOR??S EMAIL :
//---------------------------------------------------------------------
// RELEASE HISTORY
// VERSION DATE        AUTHOR  DESCRIPTION
// 1.0     2013-06-17  Slin     Created
// 1.1     2013-12-19  Slin     Modified for I/F changed.
//---------------------------------------------------------------------
// PURPOSE : Short description of functionality
//---------------------------------------------------------------------
// PARAMETERS
//     PARAM NAME      RANGE    : DESCRIPTION           : DEFAULT
//    I2C_DevADDR      [7:0]    : The device address    : 0x37
//    End_Of_ADDR      [7:0]    : End of Memory address : 0x1F
//---------------------------------------------------------------------
// REMARK
//
//
//---------------------------------------------------------------------
module i2c_csr 
#(                       
  parameter I2C_DevADDR = 7'h33 
 
)                        
(
    input   logic         clk,
    input   logic         reset_n,
    input   logic         scl_i,      // input SCL of I2C Bus.
    input   logic         sda_i,      // input SDA of I2C Bus.
    output  logic         sda_oe_n,
    input   logic  [7:0]  readdata,
    output  logic  [7:0]  writedata,
    output  logic  [7:0]  address,
    output  logic         read,
    output  logic         write
    //avl_mm_if       avl_if_m0
    );

    // Parameters
    // parameter   I2C_DevADDR = 7'h33;
    parameter   End_Of_ADDR = 8'hFF;

    // The Signals for SIPO Register
    logic       rxbuf_aclr;
    logic       rxbuf_clkenb;
    logic [7:0] rxbuf;

    // The Signals for PISO Register
    logic       txbuf_aclr;
    logic       txbuf_clkenb;
    logic       txbuf_load;
    //logic [7:0] txbuf;
    logic       txbuf_shiftout;

    // The Counter Signals for SCL
    logic       sclcnt_aclr;
    logic       sclcnt_clkenb;
    logic [3:0] scl_cntnum;

    // The Signals for I2C Bus
    logic       scl_h2l_trans;  // it's to detect the high to low transition at the SCL signal;
    logic       scl_l2h_trans;  // it's to detect the low to high transition at the SCL signal;
    //logic       scl_low;
    logic       scl_high;
    logic       sda_h2l_trans;  // it's to detect the high to low transition at the SDA signal;
    logic       sda_l2h_trans;  // it's to detect the low to high transition at the SDA signal;
    //logic       sda_low;
    logic       sda_high;
    logic       scl_sync, scl_d1, scl_d2;
    logic       sda_sync, sda_d1, sda_d2;
    logic       i2c_sta;
    logic       i2c_sto;
    logic       i2c_op; // '1' for Read, '0' for Write
    logic       i2c_m_ack;

    // The Signals for Misc.
    logic       reset_d1, reset_d2;
    logic       reset_sync;

    // The FSM is used to contrl I2C to CSRs
    enum int unsigned { stm_BusIdle, stm_StartTRX, stm_DevAddr, stm_DevAddrMatch,
                        stm_DevAddrACK, stm_RWCheck, stm_Addr, stm_AddrACK,
                        stm_PreambleRx, stm_RxData, stm_RxDataACK, stm_ContRx,
                        stm_ReadData, stm_LoadRData, stm_PreambleTx, stm_TxData,
                        stm_TxDataACK, stm_ContTx, stm_StopTRX } prev_state, curt_state;

    //-----------------------------------------------------------------
    // Call a Resync. IO Module.
    //-----------------------------------------------------------------
    // altera_std_synchronizer_bundle
        // #(
          // .width (2),
          // .depth (2)
         // )
        // I2CBUS_SYNC
        // (
         // .clk     ( clk ),
         // .reset_n ( reset_n ),
         // .din     ( {scl_i, sda_i} ),
         // .dout    ( {scl_sync, sda_sync} )
        // );

    always_ff @(posedge clk, posedge reset_sync)
    begin
        if (reset_sync) begin
            {scl_d2, scl_d1} <= {scl_i, scl_i};
            {sda_d2, sda_d1} <= {sda_i, sda_i};
        end
        else begin
            {scl_d2, scl_d1} <= {scl_d1, scl_i};
            {sda_d2, sda_d1} <= {sda_d1, sda_i};
        end
    end

    //-----------------------------------------------------------------
    // Regenerates a sync. reset signal.
    //-----------------------------------------------------------------
    always_ff @(posedge clk, negedge reset_n)
        begin
            if (!reset_n)
                {reset_d2, reset_d1} <= '0;
            else
                {reset_d2, reset_d1} <= {reset_d1, 1'b1};
        end

    assign reset_sync = ~reset_d2;

    //-----------------------------------------------------------------
    // The SIPO Registers for I2C write operation.
    //-----------------------------------------------------------------
    sipo_8b RX_BUF (
        .aclr       (rxbuf_aclr),
        .clock      (clk),
        .enable     (rxbuf_clkenb),
        .shiftin    (sda_i),
        .q          (rxbuf)
        );

    always_ff @(posedge clk, posedge reset_sync)
        begin
            if (reset_sync)
                rxbuf_aclr = '1;
            else
                rxbuf_aclr <= ( (curt_state == stm_BusIdle) ||
                                (curt_state == stm_RWCheck) ||
                                (curt_state == stm_PreambleRx)
                               ) ;
        end

    always_ff @(posedge clk, posedge reset_sync)
        begin
            if (reset_sync)
                rxbuf_clkenb <= '0;
            else
                rxbuf_clkenb <= ( ((curt_state == stm_DevAddr) && scl_l2h_trans) ||
                                  ((curt_state == stm_Addr) && scl_l2h_trans) ||
                                  ((curt_state == stm_RxData) && scl_l2h_trans)
                                 );
        end

    //-----------------------------------------------------------------
    // The PISO Registers for I2C read operation.
    //-----------------------------------------------------------------
    piso_8b TX_BUF (
        .aclr       (txbuf_aclr),
        .clock      (clk),
        .data       (readdata),
        .enable     (txbuf_clkenb),
        .load       (txbuf_load),
        .shiftout   (txbuf_shiftout)
        );

    always_ff @(posedge clk, posedge reset_sync)
        begin
            if (reset_sync)
                txbuf_aclr <= '1;
            else
                txbuf_aclr <= ((curt_state == stm_BusIdle) ||
                               (curt_state == stm_ReadData)) ;
        end

    always_ff @(posedge clk, posedge reset_sync)
        begin
            if (reset_sync)
                txbuf_clkenb <= '0;
            else
                txbuf_clkenb <= ( //(curt_state == stm_PreambleTx) ||
                                  (curt_state == stm_LoadRData) ||
                                  ((curt_state == stm_TxData) && scl_h2l_trans)
                                 );
        end

    always_ff @(posedge clk, posedge reset_sync)
        begin
            if (reset_sync)
                txbuf_load <= '0;
            else
                txbuf_load <= (curt_state == stm_LoadRData);
        end

    //-----------------------------------------------------------------
    // The counter is used to conut amount of SCL.
    //-----------------------------------------------------------------
    cnt4b_up CNT_SCL (
        .aclr   (sclcnt_aclr),
        .clk_en (sclcnt_clkenb),
        .clock  (clk),
        .q      (scl_cntnum)
        );

    always_ff @(posedge clk, posedge reset_sync)
        begin
            if (reset_sync)
                sclcnt_aclr <= '1;
            else
                sclcnt_aclr <= ( (curt_state == stm_BusIdle) ||
                                 (curt_state == stm_RWCheck) ||
                                 (curt_state == stm_PreambleRx) ||
                                 (curt_state == stm_ReadData)
                                );
        end

    always_ff @(posedge clk, posedge reset_sync)
        begin
            if (reset_sync)
                sclcnt_clkenb <= '0;
            else
                sclcnt_clkenb <= ( scl_h2l_trans &&
                                   ( (curt_state == stm_DevAddr) ||
                                     (curt_state == stm_DevAddrACK) ||
                                     (curt_state == stm_Addr) ||
                                     (curt_state == stm_AddrACK) ||
                                     (curt_state == stm_RxData) ||
                                     (curt_state == stm_RxDataACK) ||
                                     (curt_state == stm_TxData) ||
                                     (curt_state == stm_TxDataACK) )
                                   );
        end

    //-----------------------------------------------------------------
    // The Memory-Mapping Bus Statment
    //-----------------------------------------------------------------
    always_ff @(posedge clk, posedge reset_sync)
        begin
            if (reset_sync)
                read <= '0;
            else
                read <= (curt_state == stm_ReadData);
        end

    always_ff @(posedge clk, posedge reset_sync)
        begin
            if (reset_sync)
                write <= '0;
            else
                write <= ( (curt_state == stm_RxDataACK) &&
                                     (scl_cntnum == 4'd9) );
        end

    always_ff @(posedge clk, posedge reset_sync)
        begin
            if (reset_sync)
                address <= '0;
            else begin
                if ( i2c_sto == 1'b1 )
                    address <= '0;
                else if ( (curt_state == stm_Addr) && (scl_cntnum == 4'd8) )
                    address <= rxbuf;
                else if ( ((curt_state == stm_ContRx) && (address != End_Of_ADDR)) ||
                      ((curt_state == stm_ContTx) && (!i2c_m_ack)) )
                    address <= address + 5'd1;
            end
        end

    always_ff @(posedge clk, posedge reset_sync)
        begin
            if (reset_sync)
                writedata <= '0;
            else begin
               if ( i2c_sto == 1'b1 )
                writedata <= '0;
               else  if ( (curt_state == stm_RxData) && (scl_cntnum == 4'd8) )
                writedata <= rxbuf;
            end
        end

    //-----------------------------------------------------------------
    // I2C signals
    //-----------------------------------------------------------------
    assign scl_h2l_trans = (~scl_d1 & scl_d2);
    assign scl_l2h_trans = (scl_d1 & ~scl_d2);
    //assign scl_low       = (~scl_d1 & ~scl_d2);
    assign scl_high      = (scl_d1 & scl_d2);

    assign sda_h2l_trans = (~sda_d1 & sda_d2);
    assign sda_l2h_trans = (sda_d1 & ~sda_d2);
    //assign sda_low       = (~sda_d1 & ~sda_d2);
    assign sda_high      = (sda_d1 & sda_d2);

    assign i2c_sta = (sda_h2l_trans & scl_high);
    assign i2c_sto = (sda_l2h_trans & scl_high);

    always_ff @(posedge clk, posedge reset_sync)
        begin : I2C_OP_DET
            if (reset_sync)
                i2c_op <= '0;
            else if (curt_state == stm_DevAddrMatch)
                i2c_op <= rxbuf[0];
        end : I2C_OP_DET

    always_ff @(posedge clk, posedge reset_sync)
        begin : I2C_Master_ACK
            if (reset_sync)
                i2c_m_ack <= '1;    // the default is no acknowledgement on Master.
            else if ((curt_state == stm_TxDataACK) && scl_l2h_trans)
                i2c_m_ack <= sda_i;
        end : I2C_Master_ACK

    always_ff @(posedge clk, posedge reset_sync)
        begin
            if (reset_sync)
                sda_oe_n <= '1;    // The SDA signal is released.
            else if (curt_state == stm_TxData)
                sda_oe_n <= txbuf_shiftout;
            else if ( (curt_state == stm_DevAddrACK) || (curt_state == stm_AddrACK) ||
                     ((curt_state == stm_RxDataACK) && (address != End_Of_ADDR)) )
                sda_oe_n <= '0;
            else
                sda_oe_n <= '1;
        end

    //-----------------------------------------------------------------
    // FSM Statement
    //-----------------------------------------------------------------
    always_ff @(posedge clk, posedge reset_sync)
        begin : FSM_state
            if (reset_sync)
                curt_state <= stm_BusIdle;
            else
                curt_state <= prev_state;
        end : FSM_state

    always_comb begin : FSM_decode
        prev_state = stm_BusIdle; // default condition
        case (curt_state)
            stm_BusIdle : begin
                if (i2c_sta)
                    prev_state = stm_StartTRX;
                else
                    prev_state = stm_BusIdle;
                end
            stm_StartTRX : begin
                if (scl_h2l_trans)
                    prev_state = stm_DevAddr;
                else
                    prev_state = stm_StartTRX;
                end
            stm_DevAddr : begin
                if (scl_cntnum < 4'd8)
                    prev_state = stm_DevAddr;
                else
                    prev_state = stm_DevAddrMatch;
                end
            stm_DevAddrMatch : begin
                if (I2C_DevADDR == rxbuf[7:1])
                    prev_state = stm_DevAddrACK;
                else
                    prev_state = stm_StopTRX;
                end
            stm_DevAddrACK : begin
                if (scl_cntnum < 4'd9)
                    prev_state = stm_DevAddrACK;
                else
                    prev_state = stm_RWCheck;
                end
            stm_RWCheck : begin
                if (i2c_op)
                    prev_state = stm_ReadData; // To Read operation
                else
                    prev_state = stm_Addr; // To write operation
                end
            stm_Addr : begin
                if (i2c_sto)
                    prev_state = stm_StopTRX;
                else if (i2c_sta)
                    prev_state = stm_StartTRX;
                else if (scl_cntnum < 4'd8)
                    prev_state = stm_Addr;
                else
                    prev_state = stm_AddrACK;
                end

            stm_AddrACK : begin
                if (scl_cntnum < 4'd9)
                    prev_state = stm_AddrACK;
                else
                    prev_state = stm_PreambleRx;
                end

            stm_PreambleRx : begin
                    prev_state = stm_RxData;
                end

            stm_RxData : begin
                if (i2c_sto)
                    prev_state = stm_StopTRX;
                else if (i2c_sta)
                    prev_state = stm_StartTRX;
                else if (scl_cntnum < 4'd8)
                    prev_state = stm_RxData;
                else
                    prev_state = stm_RxDataACK;
                end

            stm_RxDataACK : begin
                if (scl_cntnum < 4'd9)
                    prev_state = stm_RxDataACK;
                else
                    prev_state = stm_ContRx;
                end

            stm_ContRx : begin
                if (address == End_Of_ADDR)  // The I2C Bus transfers termination if the end of address at Target.
                    prev_state = stm_StopTRX;
                else
                    prev_state = stm_PreambleRx;
                end

            stm_ReadData : begin
                    prev_state = stm_LoadRData;
                end

            stm_LoadRData : begin
                    prev_state = stm_PreambleTx;
                end

            stm_PreambleTx : begin
                    prev_state = stm_TxData;
                end

            stm_TxData : begin
                if (scl_cntnum < 4'd8)
                    prev_state = stm_TxData;
                else
                    prev_state = stm_TxDataACK;
                end

            stm_TxDataACK : begin
                if (i2c_sto)
                    prev_state = stm_StopTRX;
                else if (i2c_sta)
                    prev_state = stm_StartTRX;
                else if (scl_cntnum < 4'd9)
                    prev_state = stm_TxDataACK;
                else
                    prev_state = stm_ContTx;
                end

            stm_ContTx : begin
                if (i2c_m_ack)  // If Master no acknowledgement.
                    prev_state = stm_StopTRX;
                else
                    prev_state = stm_ReadData;
                end

            stm_StopTRX : begin
                if (scl_high && sda_high)   // It's waiting for I2C Bus free.
                    prev_state = stm_BusIdle;
                else
                    prev_state = stm_StopTRX;
                end

             default : begin
                    prev_state = stm_BusIdle;
                end
        endcase
    end : FSM_decode

endmodule
