 /*
facebook wedge100
Model: wedge100 led interface
Revision: V0.1, 
History : 
2015/03/12 - initial draft by zhiping Yao
2015/06/16 - zhiping add led test logic 
*/
//-----------------------------------------------------------------------------
// Title       : led interface module
// Design      : Wedge100 CPLD Design
// File        : led_test.v
//-----------------------------------------------------------------------------
// Version     : Ver 2.0
// Changes     : 1. Removed the unused signals and rearranged the code
//             : 2. Added 1Hz clock input signal for LED_CLK0 and made LED_DATA0
//               always low
//-----------------------------------------------------------------------------
// Version     : Ver 2.1
// Changes     : 1. Instantiate the LED shift register implementation module
//-----------------------------------------------------------------------------
// Description:- LED interface implementation
//-----------------------------------------------------------------------------
module led_if
(
          
  input	         hot_reset_n 	             // cold reset output, reset all devices on board, both standby and main power domain
, input	         warm_reset_n                // warm reset output, reset standby power domain                          	       
, input	         cold_reset_n                // hot reset output, reset BMC only, will not affect datapath                     
, input          pd_stby_reset_n             // async standby power on reset, active low, from power1014a, already delayed
, input	         pd_main_reset_n             // power domain main  

, input	         clk                         // 50mhz clock input
, input          clk_10hz                    // button selection clock
, input	         clk_blink                   // blinking clock input, 500ms??
, input	         pd_main_ok                  // power domain main is OK
, input	         CPLD_QSFP_LED_POSITION      // Top/bottom selection button input           
                                             
, input  [7:0]   rw_0x3c                     // Tomahawk LED control 1
, input  [7:0]   rw_0x3d                     // Tomahawk LED control 2
, input  [7:0]   rw_0x3e                     // stream LED control 1
, input  [7:0]   rw_0x3f                     // stream LED control 2

, input  [7:0]   rw_0x46                     // Speed selection register
                                             // 00 - 100G
                                             // 01 - 50G
                                             // 1x - 25G  
											 
, input          ISO_XP_LED_CLK              // LED Stream Interface Clock input
, input          ISO_XP_LED_DAT              // LED Data, synchronized to LED_CLK input          
, input          ISO_XP_LED_STB              // LED Strobe, synchronized to LED_CLK input             
                                                                                    
, output         LED_CLK0_BUF_EN             // led stream 0 ISO buffer enable, active low
, output         LED_CLEAR0                  // led stream 0 clear
, output         LED_CLK0                    // led stream 0 clock output
, output         LED_DATA0                   // led stream 0 data output
                                                                                          
//	system LED signals                       
, output         SYS_LED1_GREEN              // system led 1, green
, output         SYS_LED1_BLUE               // system led 1, blue
, output         SYS_LED1_RED                // system led 1, red
                                             
, output         SYS_LED2_GREEN              // system led 1, green
, output         SYS_LED2_BLUE               // system led 1, blue
, output         SYS_LED2_RED                // system led 1, red

, output         top_led_active_n            //bottom qsfp selection status                                   (* chip_pin = "B4"  *)
, output         bot_led_active_n            //top qsfp selection status                                      (* chip_pin = "D6"  *) 

, output  [7:0]  ro_0x1b                     // led read only status, secreate cmd  

) ;

// {RED, Blue, Green }
parameter [2:0]   COLOR_RED    = 3'b100 ;
parameter [2:0]   COLOR_BLUE   = 3'b010 ;
parameter [2:0]   COLOR_GREEN  = 3'b001 ;
parameter [2:0]   COLOR_YELLOW = 3'b101 ; //Red, Green
parameter [2:0]   COLOR_PINK   = 3'b110 ; //Red, blue
parameter [2:0]   COLOR_AQUA   = 3'b011 ; //green, blue
parameter [2:0]   COLOR_WHITE  = 3'b111 ;
parameter [2:0]   COLOR_OFF    = 3'b000 ;

// led stream pulse generation
reg        test_led_clk;
reg [11:0] led_test_cnt;
reg [1:0]  test_led_data_act;
reg [3:0]  blink_counter;
reg [5:0]  led_test_numb_cnt;
reg        test_led_green, test_led_blue,test_led_red ; 

wire   rst_n = cold_reset_n & warm_reset_n ;
// test control reg 0x3d

wire  led_test_green;   // indicate color of the led for testing purpose
wire  led_test_blue ;   // indicate color of the led for testing purpose
wire  led_test_red  ;   // indicate color of the led for testing purpose

// led_test_cnt[7:3]: 32 LED in one stream, 4 LED(tri-color) in one colunm, 8 LED in one 2x2 qsfp28 connector
wire  [2:0] test_led_drive_color ;
wire  [2:0] test_led_spectrum_color ;
wire  [2:0] test_led_spectrum_color2 ;
wire  [2:0] test_led_spectrum_color3 ;
wire  [2:0] test_led_spectrum_color4 ;
wire  [2:0] test_led_spectrum_color5 ;
wire  [2:0] test_led_spectrum_color6 ;
wire  [2:0] test_led_spectrum_color7 ;
wire  [2:0] test_led_spectrum_color8 ;
wire  [2:0] test_led_spectrum_blink  ;
wire        test_led_page;
wire [2:0]  test_led_page_color;
 
wire [1:0] test_led_out;
wire [4:0] led_test_num ;
wire        walk_stream_select;

wire        led_test_mode_en; 
wire [2:0]  led_test_mode;  
wire        walk_test_en; 
wire       led_blink_pul;
wire       top_led_active_n_out;
wire       bot_led_active_n_out;


assign walk_stream_select = led_test_numb_cnt[5];
assign led_test_mode_en   = rw_0x3c[7]; 
assign led_test_mode      = rw_0x3c[6:4];  
assign walk_test_en       = rw_0x3c[3]; 

assign  test_led_spectrum_color  = (led_test_cnt[7:5] == 3'b000 ) ? COLOR_RED    :
                                   (led_test_cnt[7:5] == 3'b001 ) ? COLOR_BLUE   :
                                   (led_test_cnt[7:5] == 3'b010 ) ? COLOR_GREEN  :
                                   (led_test_cnt[7:5] == 3'b011 ) ? COLOR_YELLOW :
                                   (led_test_cnt[7:5] == 3'b100 ) ? COLOR_PINK   :
                                   (led_test_cnt[7:5] == 3'b101 ) ? COLOR_AQUA   :
                                   (led_test_cnt[7:5] == 3'b110 ) ? COLOR_BLUE   :
                                   (led_test_cnt[7:5] == 3'b111 ) ? COLOR_YELLOW : COLOR_OFF;

assign  test_led_spectrum_color2 = (led_test_cnt[7:5] == 3'b000 ) ? COLOR_YELLOW :
                                   (led_test_cnt[7:5] == 3'b001 ) ? COLOR_RED    :
                                   (led_test_cnt[7:5] == 3'b010 ) ? COLOR_BLUE   :
                                   (led_test_cnt[7:5] == 3'b011 ) ? COLOR_GREEN  :
                                   (led_test_cnt[7:5] == 3'b100 ) ? COLOR_YELLOW :
                                   (led_test_cnt[7:5] == 3'b101 ) ? COLOR_PINK   :
                                   (led_test_cnt[7:5] == 3'b110 ) ? COLOR_AQUA   :
                                   (led_test_cnt[7:5] == 3'b111 ) ? COLOR_BLUE   : COLOR_OFF;
                                                                    
assign  test_led_spectrum_color3 = (led_test_cnt[7:5] == 3'b000 ) ? COLOR_BLUE   :
                                   (led_test_cnt[7:5] == 3'b001 ) ? COLOR_YELLOW :
                                   (led_test_cnt[7:5] == 3'b010 ) ? COLOR_RED    :
                                   (led_test_cnt[7:5] == 3'b011 ) ? COLOR_BLUE   :
                                   (led_test_cnt[7:5] == 3'b100 ) ? COLOR_GREEN  :
                                   (led_test_cnt[7:5] == 3'b101 ) ? COLOR_YELLOW :
                                   (led_test_cnt[7:5] == 3'b110 ) ? COLOR_PINK   :
                                   (led_test_cnt[7:5] == 3'b111 ) ? COLOR_AQUA   : COLOR_OFF;
                                                                      
                                   
assign  test_led_spectrum_color4 = (led_test_cnt[7:5] == 3'b000 ) ? COLOR_AQUA   :
                                   (led_test_cnt[7:5] == 3'b001 ) ? COLOR_BLUE   :
                                   (led_test_cnt[7:5] == 3'b010 ) ? COLOR_YELLOW :
                                   (led_test_cnt[7:5] == 3'b011 ) ? COLOR_RED    :
                                   (led_test_cnt[7:5] == 3'b100 ) ? COLOR_BLUE   :
                                   (led_test_cnt[7:5] == 3'b101 ) ? COLOR_GREEN  :
                                   (led_test_cnt[7:5] == 3'b110 ) ? COLOR_YELLOW :
                                   (led_test_cnt[7:5] == 3'b111 ) ? COLOR_PINK   : COLOR_OFF;                                                                      
                                                                        
assign  test_led_spectrum_color5 = (led_test_cnt[7:5] == 3'b000 ) ? COLOR_PINK   :
                                   (led_test_cnt[7:5] == 3'b001 ) ? COLOR_AQUA   :
                                   (led_test_cnt[7:5] == 3'b010 ) ? COLOR_BLUE   :
                                   (led_test_cnt[7:5] == 3'b011 ) ? COLOR_YELLOW :
                                   (led_test_cnt[7:5] == 3'b100 ) ? COLOR_RED    :
                                   (led_test_cnt[7:5] == 3'b101 ) ? COLOR_BLUE   :
                                   (led_test_cnt[7:5] == 3'b110 ) ? COLOR_GREEN  :
                                   (led_test_cnt[7:5] == 3'b111 ) ? COLOR_YELLOW : COLOR_OFF;
                                                                      
assign  test_led_spectrum_color6 = (led_test_cnt[7:5] == 3'b000 ) ? COLOR_YELLOW :
                                   (led_test_cnt[7:5] == 3'b001 ) ? COLOR_PINK   :
                                   (led_test_cnt[7:5] == 3'b010 ) ? COLOR_AQUA   :
                                   (led_test_cnt[7:5] == 3'b011 ) ? COLOR_BLUE   :
                                   (led_test_cnt[7:5] == 3'b100 ) ? COLOR_YELLOW :
                                   (led_test_cnt[7:5] == 3'b101 ) ? COLOR_RED    :
                                   (led_test_cnt[7:5] == 3'b110 ) ? COLOR_BLUE   :
                                   (led_test_cnt[7:5] == 3'b111 ) ? COLOR_GREEN  : COLOR_OFF;
                                                                    
                                   
assign  test_led_spectrum_color7 = (led_test_cnt[7:5] == 3'b000 ) ? COLOR_GREEN  :
                                   (led_test_cnt[7:5] == 3'b001 ) ? COLOR_YELLOW :
                                   (led_test_cnt[7:5] == 3'b010 ) ? COLOR_PINK   :
                                   (led_test_cnt[7:5] == 3'b011 ) ? COLOR_AQUA   :
                                   (led_test_cnt[7:5] == 3'b100 ) ? COLOR_BLUE   :
                                   (led_test_cnt[7:5] == 3'b101 ) ? COLOR_YELLOW :
                                   (led_test_cnt[7:5] == 3'b110 ) ? COLOR_RED    :
                                   (led_test_cnt[7:5] == 3'b111 ) ? COLOR_BLUE   : COLOR_OFF;        

assign  test_led_spectrum_color8 = (led_test_cnt[7:5] == 3'b000 ) ? COLOR_BLUE  :            
                                   (led_test_cnt[7:5] == 3'b001 ) ? COLOR_GREEN  :                                                                                
                                   (led_test_cnt[7:5] == 3'b010 ) ? COLOR_YELLOW :                                              
                                   (led_test_cnt[7:5] == 3'b011 ) ? COLOR_PINK   :                                              
                                   (led_test_cnt[7:5] == 3'b100 ) ? COLOR_AQUA   :                                              
                                   (led_test_cnt[7:5] == 3'b101 ) ? COLOR_BLUE   :                                              
                                   (led_test_cnt[7:5] == 3'b110 ) ? COLOR_YELLOW :                                              
                                   (led_test_cnt[7:5] == 3'b111 ) ? COLOR_RED    : COLOR_OFF;                                   
                                                                      
// spectrum_color blinking

assign  test_led_spectrum_blink  = (blink_counter[2:0] == 3'b000 ) ? test_led_spectrum_color  :           
                                   (blink_counter[2:0] == 3'b001 ) ? test_led_spectrum_color2 :           
                                   (blink_counter[2:0] == 3'b010 ) ? test_led_spectrum_color3 :           
                                   (blink_counter[2:0] == 3'b011 ) ? test_led_spectrum_color4 :           
                                   (blink_counter[2:0] == 3'b100 ) ? test_led_spectrum_color5 :           
                                   (blink_counter[2:0] == 3'b101 ) ? test_led_spectrum_color6 :           
                                   (blink_counter[2:0] == 3'b110 ) ? test_led_spectrum_color7 :           
                                   (blink_counter[2:0] == 3'b111 ) ? test_led_spectrum_color8 : COLOR_OFF;
                                  
                                                                                                          
assign  led_test_green  = test_led_drive_color[0] ;  
assign  led_test_blue   = test_led_drive_color[1] ;  
assign  led_test_red    = test_led_drive_color[2] ; 

assign  test_led_drive_color  =  (led_test_mode[2:0] == 3'b000 ) ? rw_0x3d[7:5]            :
                                 (led_test_mode[2:0] == 3'b001 ) ? rw_0x3d[7:5]            :
                                 (led_test_mode[2:0] == 3'b010 ) ? test_led_page_color     :
                                 (led_test_mode[2:0] == 3'b011 ) ? test_led_spectrum_color : 
                                 (led_test_mode[2:0] == 3'b100 ) ? rw_0x3d[7:5]            :
                                 (led_test_mode[2:0] == 3'b101 ) ? rw_0x3d[7:5]            :
                                 (led_test_mode[2:0] == 3'b110 ) ? test_led_spectrum_blink :
                                 (led_test_mode[2:0] == 3'b111 ) ? test_led_spectrum_blink : COLOR_OFF; 
                                 
                                 
   

assign  led_test_num    = walk_test_en? led_test_numb_cnt[4:0] : rw_0x3d[4:0]; // indicate one of the led of 16 qsfp port/32 tri-color led

// counter to move led slot number
always @ (posedge clk )
    begin
       // free run counter for led test timing base 
       if      (led_blink_pul == 1'b1 )  led_test_numb_cnt <= led_test_numb_cnt - 6'd1;
       else                              led_test_numb_cnt <= led_test_numb_cnt ;  
    end   



wire  pride_mode  =  (led_test_mode == 3'b010 && led_test_mode_en == 1'b1 ) ? 1'b1:      
                     (led_test_mode == 3'b011 && led_test_mode_en == 1'b1 ) ? 1'b1:      
                     (led_test_mode == 3'b110 && led_test_mode_en == 1'b1 ) ? 1'b1:      
                     (led_test_mode == 3'b111 && led_test_mode_en == 1'b1 ) ? 1'b1: 1'b0;

wire  led_0_test_en_int  = (led_test_mode == 3'b000 && led_test_mode_en == 1'b1 ) ? 1'b1: 
                           (led_test_mode == 3'b001 && led_test_mode_en == 1'b1 ) ? 1'b1:
                           (led_test_mode == 3'b010 && led_test_mode_en == 1'b1 ) ? 1'b1:
                           (led_test_mode == 3'b011 && led_test_mode_en == 1'b1 ) ? 1'b1:
                           (led_test_mode == 3'b100 && led_test_mode_en == 1'b1 ) ? 1'b1:
                           (led_test_mode == 3'b101 && led_test_mode_en == 1'b1 ) ? 1'b1:
                           (led_test_mode == 3'b110 && led_test_mode_en == 1'b1 ) ? 1'b1:
                           (led_test_mode == 3'b111 && led_test_mode_en == 1'b1 ) ? 1'b1: 1'b0;
                                                         
wire  led_1_test_en_int  = (led_test_mode == 3'b000 && led_test_mode_en == 1'b1 ) ? 1'b1:      
                           (led_test_mode == 3'b001 && led_test_mode_en == 1'b1 ) ? 1'b1:      
                           (led_test_mode == 3'b010 && led_test_mode_en == 1'b1 ) ? 1'b1:      
                           (led_test_mode == 3'b011 && led_test_mode_en == 1'b1 ) ? 1'b1:      
                           (led_test_mode == 3'b100 && led_test_mode_en == 1'b1 ) ? 1'b1:      
                           (led_test_mode == 3'b101 && led_test_mode_en == 1'b1 ) ? 1'b1:      
                           (led_test_mode == 3'b110 && led_test_mode_en == 1'b1 ) ? 1'b1:      
                           (led_test_mode == 3'b111 && led_test_mode_en == 1'b1 ) ? 1'b1: 1'b0;
                           
                           

wire  led_0_test_en   = walk_test_en ?  walk_stream_select: led_0_test_en_int ;
wire  led_1_test_en   = walk_test_en ? !walk_stream_select: led_1_test_en_int ;


// led test mode  rw_0x3c[6:4]
// 000: stream-0 single led check, led number and color is decided by register 0x3d, constant
// 001: stream-1 single led check, led number and color is decided by register 0x3d, constant
// 010: stream-0 all    led check, led color is decided by register 0x3d, constant
// 011: stream-1 all    led check, led color is decided by register 0x3d, constant
// 100: stream-0 single led check, led number and color is decided by register 0x3d, blinking 
// 101: stream-1 single led check, led number and color is decided by register 0x3d, blinking 
// 110: stream-0 all    led check, led color is decided by register 0x3d, blinking            
// 111: stream-1 all    led check, led color is decided by register 0x3d, blinking 

//wire  led_blink_en  = rw_0x3c[6] ; // blinking control           
 
// output
wire  led_0_clk_dbg   = led_0_test_en ? test_led_clk : clk ;
wire  led_1_clk_dbg   = led_1_test_en ? test_led_clk : clk ;

wire  led_0_data_dbg  = led_0_test_en ? test_led_out[0] : 1'b0 ;  // active high
wire  led_1_data_dbg  = led_1_test_en ? test_led_out[1] : 1'b0 ;  // active high


wire  led_0_clear   = !rw_0x3c[0]; 
wire  led_1_clear   = !rw_0x3c[0]; 
wire  th_led0_en    =  rw_0x3c[1];
wire  th_led1_en    =  rw_0x3c[1];
 
wire  led_0_data_int  = 1'b0 ;   // low all the time


// assign  LED_CLEAR0       =  led_0_clear & rst_n;
// assign  LED_CLK0         =  clk_blink; // Clock of 1Hz frequency
// assign  LED_DATA0        =  led_0_data_int;
// assign  LED_CLK0_BUF_EN  =  pd_main_ok ; //ICS8312 clock buffer OE is active high
// assign  LED_CLEAR1       =  led_1_clear & rst_n;
// assign  LED_CLK1         =  led_1_clk_int;
// assign  LED_DATA1        =  led_1_data_int;
// assign  LED_CLK1_BUF_EN  =  pd_main_ok ; //ICS8312 clock buffer OE is active high

/*
 tri-color LED 1
 rw_0x20[2:0]  : R, G, B => 0: OFF, 1: ON
 rw-0x20[3]    : Blink Enable, 0: no blink; 1: blink

tri-color LED 2
 rw_0x21[2:0]  : R, G, B => 0: OFF, 1: ON
 rw-0x21[3]    : Blink Enable, 0: no blink; 1: blink


*/

// drive system led

wire  led_1_red   =  rw_0x3e[3]? (clk_blink & rw_0x3e[0]) :rw_0x3e[0]  ;
wire  led_1_green =  rw_0x3e[3]? (clk_blink & rw_0x3e[1]) :rw_0x3e[1]  ;
wire  led_1_blue  =  rw_0x3e[3]? (clk_blink & rw_0x3e[2]) :rw_0x3e[2]  ;
                                 
wire  led_2_red   =  rw_0x3f[3]? (clk_blink & rw_0x3f[0]) :rw_0x3f[0]  ;
wire  led_2_green =  rw_0x3f[3]? (clk_blink & rw_0x3f[1]) :rw_0x3f[1]  ;
wire  led_2_blue  =  rw_0x3f[3]? (clk_blink & rw_0x3f[2]) :rw_0x3f[2]  ;


wire  brd_normal = pd_main_ok & warm_reset_n & cold_reset_n & pd_stby_reset_n & pd_main_reset_n & hot_reset_n ;

// LED is active low
// assign  SYS_LED1_RED    = (brd_normal) ? !led_1_red   : pd_main_ok ;                            	     
// assign  SYS_LED1_GREEN  = (brd_normal) ? !led_1_green : warm_reset_n;                                         
// assign  SYS_LED1_BLUE   = (brd_normal) ? !led_1_blue  : cold_reset_n;                                         
//                                                                             
// assign  SYS_LED2_RED    = (brd_normal) ? !led_2_red   : pd_stby_reset_n ;                              
// assign  SYS_LED2_GREEN  = (brd_normal) ? !led_2_green : pd_main_reset_n;                                     
// assign  SYS_LED2_BLUE   = (brd_normal) ? !led_2_blue  : hot_reset_n;        

assign  SYS_LED1_RED    = (brd_normal) ? !led_1_red   : pd_main_ok ;      
assign  SYS_LED1_GREEN  = (brd_normal) ? !led_1_green : warm_reset_n;     
assign  SYS_LED1_BLUE   = (brd_normal) ? !led_1_blue  : cold_reset_n;     
                                                                          
assign  SYS_LED2_RED    = (brd_normal) ? !led_2_red   : pd_stby_reset_n ; 
assign  SYS_LED2_GREEN  = (brd_normal) ? !led_2_green : pd_main_reset_n;  
assign  SYS_LED2_BLUE   = (brd_normal) ? !led_2_blue  : hot_reset_n;      

                             
                                                                             
//=================================                                          
assign  top_led_active_n  = top_led_active_n_out ;                           
assign  bot_led_active_n  = bot_led_active_n_out ; 

assign  ro_0x1b[0]  =  top_led_active_n_out ; 
assign  ro_0x1b[1]  =  bot_led_active_n_out ; 



led_selection led_selection_inst 
(
  .clk		                (clk_10hz                ),		     // clock                                                                                               
  .CPLD_QSFP_LED_POSITION   (CPLD_QSFP_LED_POSITION  ),          // LED selection button input
                                                       
  .top_led_active_n         (top_led_active_n_out    ),          // led select
  .bot_led_active_n         (bot_led_active_n_out    )
  
) ;

// led shift test logic
// shifting sequence: green, blue, red 

//======================================================================================= 

// led test trigger pulse, 0 -> 1 rising edge

wire   led_start_pul;
pos_edge	led_0_start_pos_edge        //  
(
.clk	 		(clk),
.rst_n	 		(rst_n),
.sig_in     	(clk_10hz),      // every 100ms generate one pulse
.pos_edge_en	(led_start_pul) 
) ;


pos_edge	led_blink_pos_edge        //  
(
.clk	 		(clk),
.rst_n	 		(rst_n),
.sig_in     	(clk_blink),      // every 1s generate one pulse
.pos_edge_en	(led_blink_pul) 
) ;

// generate blinking control signal

 
always @ (posedge clk or negedge rst_n)
    begin
        
       //
       if      (!rst_n)                   blink_counter <=  4'd0 ;
       else if (led_blink_pul == 1'b1 )   blink_counter <=  blink_counter + 4'd1 ; 
       
    end   
 
wire   blink_active = blink_counter[0];


 
assign   test_led_page        = blink_counter[0];

assign   test_led_page_color  = (blink_counter[2:0] == 3'b000 ) ? COLOR_RED    :
                                (blink_counter[2:0] == 3'b001 ) ? COLOR_BLUE   :
                                (blink_counter[2:0] == 3'b010 ) ? COLOR_GREEN  :
                                (blink_counter[2:0] == 3'b011 ) ? COLOR_YELLOW :
                                (blink_counter[2:0] == 3'b100 ) ? COLOR_PINK   :
                                (blink_counter[2:0] == 3'b101 ) ? COLOR_AQUA   :
                                (blink_counter[2:0] == 3'b110 ) ? COLOR_BLUE   :
                                (blink_counter[2:0] == 3'b111 ) ? COLOR_YELLOW : COLOR_OFF;
                                   

 
always @ (posedge clk )
    begin
       // free run counter for led test timing base 
       if      (led_start_pul == 1'b1 )    led_test_cnt <=  12'd0;
       else if (led_test_cnt  == 12'hFFF)  led_test_cnt <= led_test_cnt ;
       else                                led_test_cnt <= led_test_cnt + 12'd1;
       //
         
     // led test pulse generation, each stream has 16 ports, 32 tri-color led, 32x3=96bit. 
     // use 256bit counter, every two bits is a single color led. last two bits not used in 8bit cycle
     // low 8bits for 3 led: bit0,1 for green, bit2,3 for blue, bit4,5 for red, bit6,7 unused
     // led_clk is when cnt[0] = 1
       // if   ( led_test_cnt == 8'd0 )                          test_led_act  <= 1'b1;
	   // else ( led_test_cnt == {led_test_num[4:0], 3'b111 } )  test_led_act  <= 1'b0;
	 
	   // led clock generation, two bits is one led_clk cycle, 256 base, 96 clock cycle
	   if       ( led_test_cnt[2:0] == 3'b001 ) test_led_clk <= 1'b1; // red led clock                             
	   else if  ( led_test_cnt[2:0] == 3'b011 ) test_led_clk <= 1'b1; // blue  led clock                             
	   else if  ( led_test_cnt[2:0] == 3'b101 ) test_led_clk <= 1'b1; // green   led clock                             
	   else                                     test_led_clk <= 1'b0;                                                
	                                                                                                                 
	   // Green, Blue, Red                                                                                           
	   if   ( led_test_cnt[2:1] == 2'b10 )  test_led_green  <= led_test_green;                                       
	   else                                 test_led_green  <= 1'b0;                                                 
                                                                                                                     
	   if   ( led_test_cnt[2:1] == 2'b01 )  test_led_blue   <= led_test_blue;
	   else                                 test_led_blue   <= 1'b0;
	   
	   if   ( led_test_cnt[2:1] == 2'b00 )  test_led_red    <= led_test_red;
	   else                                 test_led_red    <= 1'b0;
	   
	   // led position match. 
	   // led test mode  rw_0x3c[6:4]                                                                
       // 000: stream-0 single led check, led number and color is decided by register 0x3d, constant 
       // 001: stream-1 single led check, led number and color is decided by register 0x3d, constant 
       // 010: stream-0 all    led check, led color is decided by register 0x3d, constant            
       // 011: stream-1 all    led check, led color is decided by register 0x3d, constant            
       // 100: stream-0 single led check, led number and color is decided by register 0x3d, blinking 
       // 101: stream-1 single led check, led number and color is decided by register 0x3d, blinking 
       // 110: stream-0 all    led check, led color is decided by register 0x3d, blinking            
       // 111: stream-1 all    led check, led color is decided by register 0x3d, blinking            

	   if       ( walk_test_en == 1'b0 && led_test_mode == 3'b000  &&  led_test_cnt[7:3] == led_test_num[4:0] )                   test_led_data_act <= {test_led_page, !test_led_page } ; // alternate stream 0 to 1, and back to 0
	   else if  ( walk_test_en == 1'b0 && led_test_mode == 3'b001  &&  led_test_cnt[7:3] == led_test_num[4:0] )                   test_led_data_act <= 2'b11 ;  // both stream at the same time
	   else if  ( walk_test_en == 1'b0 && led_test_mode == 3'b010 )                                                               test_led_data_act <= 2'b11 ;  // pride mode
	   else if  ( walk_test_en == 1'b0 && led_test_mode == 3'b011 )                                                               test_led_data_act <= 2'b11 ;
	   else if  ( walk_test_en == 1'b0 && led_test_mode == 3'b100  &&  led_test_cnt[7:3] == led_test_num[4:0] && blink_active )   test_led_data_act <= {test_led_page, !test_led_page } ;
	   else if  ( walk_test_en == 1'b0 && led_test_mode == 3'b101  &&  led_test_cnt[7:3] == led_test_num[4:0] && blink_active )   test_led_data_act <= 2'b11 ;
	   else if  ( walk_test_en == 1'b0 && led_test_mode == 3'b110  )                                                              test_led_data_act <= 2'b11 ;
	   else if  ( walk_test_en == 1'b0 && led_test_mode == 3'b111  &&  blink_active)                                              test_led_data_act <= 2'b11 ;
	   else if  ( walk_test_en == 1'b1 && walk_stream_select == 1'b1 && led_test_cnt[7:3] == led_test_num[4:0] )                  test_led_data_act <= 2'b01 ;
	   else if  ( walk_test_en == 1'b1 && walk_stream_select == 1'b0 && led_test_cnt[7:3] == led_test_num[4:0] )                  test_led_data_act <= 2'b10 ;
	                                                                                                                                                         
	   else                                                                                                                      test_led_data_act <= 2'b00 ;
	   
	   
	   
    end
 
assign  test_led_out[0] = ( test_led_green || test_led_blue || test_led_red ) && test_led_data_act[0] ; 
assign  test_led_out[1] = ( test_led_green || test_led_blue || test_led_red ) && test_led_data_act[1] ;

// Instantiate the LED stream Shift register module
  led_sr_if led_sr_if_inst(
   .clk             (clk ),
   .rst             (cold_reset_n ),
   .speed_sel       (rw_0x46[1:0]),         
   .top_bottom_sel  ({bot_led_active_n_out,top_led_active_n}),
   .ISO_XP_LED_CLK  (ISO_XP_LED_CLK ),
   .ISO_XP_LED_DAT  (ISO_XP_LED_DAT ),
   .ISO_XP_LED_STB  (ISO_XP_LED_STB ),
   .LED_CLK0_BUF_EN (LED_CLK0_BUF_EN),
   .LED_CLK0        (LED_CLK0       ),
   .LED_CLEAR0      (LED_CLEAR0     ),
   .LED_DATA0       (LED_DATA0      )
  );

endmodule		     