 /*
facebook wedge100
Model: wedge100 led interface
Revision: V0.1,
History :
2015/03/12 - initial draft by zhiping Yao
2015/06/16 - zhiping add led test logic
*/
//-----------------------------------------------------------------------------
// Title       : led interface module
// Design      : Wedge100 CPLD Design
// File        : led_sr_if.v
//-----------------------------------------------------------------------------
// Version     : Ver 2.0
// Changes     : 1. Removed the unused signals and rearranged the code
//             : 2. Added 1Hz clock input signal for LED_CLK0 and made LED_DATA0
//               always low
//-----------------------------------------------------------------------------
// Version     : Ver 2.1
// Changes     : 1. implementated the LED shift register and LED speed selection
//-----------------------------------------------------------------------------
// Description:- LED shift register interface implementation
//-----------------------------------------------------------------------------
`timescale 1ns/1ns

module led_sr_if
(
  input          clk             , // 50mhz clock input
  input          rst             , // 50mhz clock input
  input  [1:0]   speed_sel       , // LED Speed Selection
                                   // 00 - 100G
                                   // 01 - 50G
                                   // 1x - 25G
  input  [1:0]   top_bottom_sel  , // Top/Bottom LED stream selection for 25G
                                   // 0 -> Top 25G
                                   // 1 -> Bottom 25G

  input          ISO_XP_LED_CLK  , // LED Stream Interface Clock input
  input          ISO_XP_LED_DAT  , // LED Data, synchronized to LED_CLK input
  input          ISO_XP_LED_STB  , // LED Strobe, synchronized to LED_CLK input

  output    reg  LED_CLK0_BUF_EN , // led stream 0 ISO buffer enable
  output    reg  LED_CLK0        , // LED Stream 0 Interface Clock output
  output    reg  LED_CLEAR0      , // led stream 0 clear
  output    reg  LED_DATA0         // led stream 0 data output
);

/**********************************************************************
*                     Parameter Declarations                          *
**********************************************************************/
  // FSM states
  parameter IDLE        = 0;
  parameter CLK_NEG     = 1;
  parameter CLK_POS     = 2;

/**********************************************************************
*                     Register and wire Declarations                  *
**********************************************************************/
  reg            ISO_XP_LED_CLK_D1 ;
  reg            ISO_XP_LED_DAT_D1 ;
  reg            ISO_XP_LED_STB_D1 ;
  reg            ISO_XP_LED_CLK_D2 ;
  reg            ISO_XP_LED_DAT_D2 ;
  reg            ISO_XP_LED_STB_D2 ;
  reg            ISO_XP_LED_CLK_D3 ;
  reg   [2:0]    bit_counter       ;
  reg   [7:0]    stream_counter    ;
  reg   [127:0]  stream_cap_sr     ;
  reg   [7:0]    led_sr_counter    ;
  reg            stream_cap_done   ;
  reg            link_act          ;
  reg   [191:0]  led_sr            ;
  reg   [1:0]    sr_state          ;


  // Synchronize LED Stream Interface signals
  always @(posedge clk or negedge rst) begin
    if (rst == 1'b0) begin
      ISO_XP_LED_CLK_D1 <= 1'b0;
      ISO_XP_LED_DAT_D1 <= 1'b0;
      ISO_XP_LED_STB_D1 <= 1'b0;
      ISO_XP_LED_CLK_D2 <= 1'b0;
      ISO_XP_LED_DAT_D2 <= 1'b0;
      ISO_XP_LED_STB_D2 <= 1'b0;
      ISO_XP_LED_CLK_D3 <= 1'b0;
    end
    else begin
      ISO_XP_LED_CLK_D1 <= ISO_XP_LED_CLK;
      ISO_XP_LED_DAT_D1 <= ISO_XP_LED_DAT;
      ISO_XP_LED_STB_D1 <= ISO_XP_LED_STB;
      ISO_XP_LED_CLK_D2 <= ISO_XP_LED_CLK_D1;
      ISO_XP_LED_DAT_D2 <= ISO_XP_LED_DAT_D1;
      ISO_XP_LED_STB_D2 <= ISO_XP_LED_STB_D1;
      ISO_XP_LED_CLK_D3 <= ISO_XP_LED_CLK_D2;
    end
  end

  // Link and Activity bit capture logic
  always @(posedge clk or negedge rst) begin
    if (rst == 1'b0) begin
      bit_counter     <= 'h0;
      stream_counter  <= 'h0;
      stream_cap_sr   <= 'h0;
      stream_cap_done <= 1'b0;
    end
    else if (ISO_XP_LED_CLK_D2 == 1'b0 && ISO_XP_LED_CLK_D3 == 1'b1) begin
    // Falling edge of LED Clock
      if (ISO_XP_LED_STB_D2 == 1'b1) begin
      // Shift register and counters are reset to zero when Strobe signal received
        stream_counter  <= 'h0;
        stream_cap_sr   <= 'h0;
        stream_cap_done <= 1'b0;
        link_act        <= ISO_XP_LED_DAT_D2; // Latch first bit
        bit_counter     <= 'h1; // Load the bit counter with 1,
                                // since first bit received along with Strobe signal
      end
      else begin
        if (bit_counter == 3'd4) begin
          bit_counter <= 'h0;
          stream_cap_sr <= {stream_cap_sr[126:0], link_act};
          if (stream_counter == 8'd127) begin
            stream_counter  <= 'h0;
            stream_cap_done <= 1'b1;
          end
          else begin
            stream_counter <= stream_counter + 'h1;
          end
        end
        else begin
          bit_counter <= bit_counter + 'h1;
          case (bit_counter)
            3'h0: begin
              link_act <= ISO_XP_LED_DAT_D2;
            end
            3'h1: begin
              link_act <= ~link_act | (link_act & ISO_XP_LED_DAT_D2);
            end
          endcase
        end
      end
    end
    else begin
      stream_cap_done <= 1'b0;
    end
  end

  // LED Shift register Logic implementation
  always @(posedge clk or negedge rst) begin
    if (rst == 1'b0) begin
      led_sr            <= 'h0;
      sr_state          <= IDLE;
      LED_CLK0          <= 1'b0;
      LED_DATA0         <= 1'b0;
      led_sr_counter    <= 8'b0;
    end
    else begin
      case (sr_state)
        IDLE: begin
          LED_CLK0          <= 1'b0;
          LED_DATA0         <= 1'b0;
          led_sr_counter    <= 8'b0;
          if (stream_cap_done) begin
            sr_state   <= CLK_NEG;
             casex ({top_bottom_sel, speed_sel})
               4'bxx_00: begin // 100G
                 led_sr <= {1'b1,stream_cap_sr[4],2'b11,stream_cap_sr[0],7'h7F,
                            1'b1,stream_cap_sr[8],2'b11,stream_cap_sr[12],7'h7F,
                            1'b1,stream_cap_sr[20],2'b11,stream_cap_sr[16],7'h7F,
                            1'b1,stream_cap_sr[28],2'b11,stream_cap_sr[24],7'h7F,
                            1'b1,stream_cap_sr[36],2'b11,stream_cap_sr[32],7'h7F,
                            1'b1,stream_cap_sr[40],2'b11,stream_cap_sr[44],7'h7F,
                            1'b1,stream_cap_sr[48],2'b11,stream_cap_sr[52],7'h7F,
                            1'b1,stream_cap_sr[56],2'b11,stream_cap_sr[60],7'h7F,
                            1'b1,stream_cap_sr[76],2'b11,stream_cap_sr[72],7'h7F,
                            1'b1,stream_cap_sr[68],2'b11,stream_cap_sr[64],7'h7F,
                            1'b1,stream_cap_sr[92],2'b11,stream_cap_sr[88],7'h7F,
                            1'b1,stream_cap_sr[84],2'b11,stream_cap_sr[80],7'h7F,
                            1'b1,stream_cap_sr[108],2'b11,stream_cap_sr[104],7'h7F,
                            1'b1,stream_cap_sr[100],2'b11,stream_cap_sr[96],7'h7F,
                            1'b1,stream_cap_sr[120],2'b11,stream_cap_sr[124],7'h7F,
                            1'b1,stream_cap_sr[112],2'b11,stream_cap_sr[116],7'h7F};
               end
               4'bxx_01: begin // 50G
                 led_sr <= {stream_cap_sr[4],stream_cap_sr[4],1'b1,stream_cap_sr[0],stream_cap_sr[0],1'b1,
                            stream_cap_sr[6],stream_cap_sr[6],1'b1,stream_cap_sr[2],stream_cap_sr[2],1'b1,
                            stream_cap_sr[8],stream_cap_sr[8],1'b1,stream_cap_sr[12],stream_cap_sr[12],1'b1,
                            stream_cap_sr[10],stream_cap_sr[10],1'b1,stream_cap_sr[14],stream_cap_sr[14],1'b1,
                            stream_cap_sr[20],stream_cap_sr[20],1'b1,stream_cap_sr[16],stream_cap_sr[16],1'b1,
                            stream_cap_sr[22],stream_cap_sr[22],1'b1,stream_cap_sr[18],stream_cap_sr[18],1'b1,
                            stream_cap_sr[28],stream_cap_sr[28],1'b1,stream_cap_sr[24],stream_cap_sr[24],1'b1,
                            stream_cap_sr[30],stream_cap_sr[30],1'b1,stream_cap_sr[26],stream_cap_sr[26],1'b1,
                            stream_cap_sr[36],stream_cap_sr[36],1'b1,stream_cap_sr[32],stream_cap_sr[32],1'b1,
                            stream_cap_sr[38],stream_cap_sr[38],1'b1,stream_cap_sr[34],stream_cap_sr[34],1'b1,
                            stream_cap_sr[40],stream_cap_sr[40],1'b1,stream_cap_sr[44],stream_cap_sr[44],1'b1,
                            stream_cap_sr[42],stream_cap_sr[42],1'b1,stream_cap_sr[46],stream_cap_sr[46],1'b1,
                            stream_cap_sr[48],stream_cap_sr[48],1'b1,stream_cap_sr[52],stream_cap_sr[52],1'b1,
                            stream_cap_sr[50],stream_cap_sr[50],1'b1,stream_cap_sr[54],stream_cap_sr[54],1'b1,
                            stream_cap_sr[56],stream_cap_sr[56],1'b1,stream_cap_sr[60],stream_cap_sr[60],1'b1,
                            stream_cap_sr[58],stream_cap_sr[58],1'b1,stream_cap_sr[62],stream_cap_sr[62],1'b1,
                            stream_cap_sr[76],stream_cap_sr[76],1'b1,stream_cap_sr[72],stream_cap_sr[72],1'b1,
                            stream_cap_sr[78],stream_cap_sr[78],1'b1,stream_cap_sr[74],stream_cap_sr[74],1'b1,
                            stream_cap_sr[68],stream_cap_sr[68],1'b1,stream_cap_sr[64],stream_cap_sr[64],1'b1,
                            stream_cap_sr[70],stream_cap_sr[70],1'b1,stream_cap_sr[66],stream_cap_sr[66],1'b1,
                            stream_cap_sr[92],stream_cap_sr[92],1'b1,stream_cap_sr[88],stream_cap_sr[88],1'b1,
                            stream_cap_sr[94],stream_cap_sr[94],1'b1,stream_cap_sr[90],stream_cap_sr[90],1'b1,
                            stream_cap_sr[84],stream_cap_sr[84],1'b1,stream_cap_sr[80],stream_cap_sr[80],1'b1,
                            stream_cap_sr[86],stream_cap_sr[86],1'b1,stream_cap_sr[82],stream_cap_sr[82],1'b1,
                            stream_cap_sr[108],stream_cap_sr[108],1'b1,stream_cap_sr[104],stream_cap_sr[104],1'b1,
                            stream_cap_sr[110],stream_cap_sr[110],1'b1,stream_cap_sr[106],stream_cap_sr[106],1'b1,
                            stream_cap_sr[100],stream_cap_sr[100],1'b1,stream_cap_sr[96],stream_cap_sr[96],1'b1,
                            stream_cap_sr[102],stream_cap_sr[102],1'b1,stream_cap_sr[98],stream_cap_sr[98],1'b1,
                            stream_cap_sr[120],stream_cap_sr[120],1'b1,stream_cap_sr[124],stream_cap_sr[124],1'b1,
                            stream_cap_sr[122],stream_cap_sr[122],1'b1,stream_cap_sr[126],stream_cap_sr[126],1'b1,
                            stream_cap_sr[112],stream_cap_sr[112],1'b1,stream_cap_sr[116],stream_cap_sr[116],1'b1,
                            stream_cap_sr[114],stream_cap_sr[114],1'b1,stream_cap_sr[118],stream_cap_sr[118],1'b1};
               end
               4'b01_1x: begin // Top 25G
                 led_sr <= {stream_cap_sr[4],1'b1,stream_cap_sr[4],stream_cap_sr[5],1'b1,stream_cap_sr[5],
                            stream_cap_sr[7],1'b1,stream_cap_sr[7],stream_cap_sr[6],1'b1,stream_cap_sr[6],
                            stream_cap_sr[8],1'b1,stream_cap_sr[8],stream_cap_sr[9],1'b1,stream_cap_sr[9],
                            stream_cap_sr[11],1'b1,stream_cap_sr[11],stream_cap_sr[10],1'b1,stream_cap_sr[10],
                            stream_cap_sr[20],1'b1,stream_cap_sr[20],stream_cap_sr[21],1'b1,stream_cap_sr[21],
                            stream_cap_sr[23],1'b1,stream_cap_sr[23],stream_cap_sr[22],1'b1,stream_cap_sr[22],
                            stream_cap_sr[29],1'b1,stream_cap_sr[29],stream_cap_sr[28],1'b1,stream_cap_sr[28],
                            stream_cap_sr[31],1'b1,stream_cap_sr[31],stream_cap_sr[30],1'b1,stream_cap_sr[30],
                            stream_cap_sr[36],1'b1,stream_cap_sr[36],stream_cap_sr[37],1'b1,stream_cap_sr[37],
                            stream_cap_sr[39],1'b1,stream_cap_sr[39],stream_cap_sr[38],1'b1,stream_cap_sr[38],
                            stream_cap_sr[40],1'b1,stream_cap_sr[40],stream_cap_sr[41],1'b1,stream_cap_sr[41],
                            stream_cap_sr[43],1'b1,stream_cap_sr[43],stream_cap_sr[42],1'b1,stream_cap_sr[42],
                            stream_cap_sr[48],1'b1,stream_cap_sr[48],stream_cap_sr[49],1'b1,stream_cap_sr[49],
                            stream_cap_sr[51],1'b1,stream_cap_sr[51],stream_cap_sr[50],1'b1,stream_cap_sr[50],
                            stream_cap_sr[56],1'b1,stream_cap_sr[56],stream_cap_sr[57],1'b1,stream_cap_sr[57],
                            stream_cap_sr[59],1'b1,stream_cap_sr[59],stream_cap_sr[58],1'b1,stream_cap_sr[58],
                            stream_cap_sr[76],1'b1,stream_cap_sr[76],stream_cap_sr[77],1'b1,stream_cap_sr[77],
                            stream_cap_sr[78],1'b1,stream_cap_sr[78],stream_cap_sr[79],1'b1,stream_cap_sr[79],
                            stream_cap_sr[68],1'b1,stream_cap_sr[68],stream_cap_sr[69],1'b1,stream_cap_sr[69],
                            stream_cap_sr[71],1'b1,stream_cap_sr[71],stream_cap_sr[70],1'b1,stream_cap_sr[70],
                            stream_cap_sr[92],1'b1,stream_cap_sr[92],stream_cap_sr[93],1'b1,stream_cap_sr[93],
                            stream_cap_sr[95],1'b1,stream_cap_sr[95],stream_cap_sr[94],1'b1,stream_cap_sr[94],
                            stream_cap_sr[84],1'b1,stream_cap_sr[84],stream_cap_sr[85],1'b1,stream_cap_sr[85],
                            stream_cap_sr[87],1'b1,stream_cap_sr[87],stream_cap_sr[86],1'b1,stream_cap_sr[86],
                            stream_cap_sr[108],1'b1,stream_cap_sr[108],stream_cap_sr[109],1'b1,stream_cap_sr[109],
                            stream_cap_sr[111],1'b1,stream_cap_sr[111],stream_cap_sr[110],1'b1,stream_cap_sr[110],
                            stream_cap_sr[100],1'b1,stream_cap_sr[100],stream_cap_sr[101],1'b1,stream_cap_sr[101],
                            stream_cap_sr[103],1'b1,stream_cap_sr[103],stream_cap_sr[102],1'b1,stream_cap_sr[102],
                            stream_cap_sr[120],1'b1,stream_cap_sr[120],stream_cap_sr[121],1'b1,stream_cap_sr[121],
                            stream_cap_sr[123],1'b1,stream_cap_sr[123],stream_cap_sr[122],1'b1,stream_cap_sr[122],
                            stream_cap_sr[112],1'b1,stream_cap_sr[112],stream_cap_sr[113],1'b1,stream_cap_sr[113],
                            stream_cap_sr[115],1'b1,stream_cap_sr[115],stream_cap_sr[114],1'b1,stream_cap_sr[114]};
               end
               4'b10_1x: begin // Bottom 25G
                 led_sr <= {stream_cap_sr[0],1'b1,stream_cap_sr[0],stream_cap_sr[1],1'b1,stream_cap_sr[1],
                            stream_cap_sr[3],1'b1,stream_cap_sr[3],stream_cap_sr[2],1'b1,stream_cap_sr[2],
                            stream_cap_sr[12],1'b1,stream_cap_sr[12],stream_cap_sr[13],1'b1,stream_cap_sr[13],
                            stream_cap_sr[15],1'b1,stream_cap_sr[15],stream_cap_sr[14],1'b1,stream_cap_sr[14],
                            stream_cap_sr[16],1'b1,stream_cap_sr[16],stream_cap_sr[17],1'b1,stream_cap_sr[17],
                            stream_cap_sr[19],1'b1,stream_cap_sr[19],stream_cap_sr[18],1'b1,stream_cap_sr[18],
                            stream_cap_sr[24],1'b1,stream_cap_sr[24],stream_cap_sr[25],1'b1,stream_cap_sr[25],
                            stream_cap_sr[27],1'b1,stream_cap_sr[27],stream_cap_sr[26],1'b1,stream_cap_sr[26],
                            stream_cap_sr[32],1'b1,stream_cap_sr[32],stream_cap_sr[33],1'b1,stream_cap_sr[33],
                            stream_cap_sr[35],1'b1,stream_cap_sr[35],stream_cap_sr[34],1'b1,stream_cap_sr[34],
                            stream_cap_sr[44],1'b1,stream_cap_sr[44],stream_cap_sr[45],1'b1,stream_cap_sr[45],
                            stream_cap_sr[47],1'b1,stream_cap_sr[47],stream_cap_sr[46],1'b1,stream_cap_sr[46],
                            stream_cap_sr[52],1'b1,stream_cap_sr[52],stream_cap_sr[53],1'b1,stream_cap_sr[53],
                            stream_cap_sr[55],1'b1,stream_cap_sr[55],stream_cap_sr[54],1'b1,stream_cap_sr[54],
                            stream_cap_sr[60],1'b1,stream_cap_sr[60],stream_cap_sr[61],1'b1,stream_cap_sr[61],
                            stream_cap_sr[63],1'b1,stream_cap_sr[63],stream_cap_sr[62],1'b1,stream_cap_sr[62],
                            stream_cap_sr[72],1'b1,stream_cap_sr[72],stream_cap_sr[73],1'b1,stream_cap_sr[73],
                            stream_cap_sr[75],1'b1,stream_cap_sr[75],stream_cap_sr[74],1'b1,stream_cap_sr[74],
                            stream_cap_sr[64],1'b1,stream_cap_sr[64],stream_cap_sr[65],1'b1,stream_cap_sr[65],
                            stream_cap_sr[67],1'b1,stream_cap_sr[67],stream_cap_sr[66],1'b1,stream_cap_sr[66],
                            stream_cap_sr[88],1'b1,stream_cap_sr[88],stream_cap_sr[89],1'b1,stream_cap_sr[89],
                            stream_cap_sr[91],1'b1,stream_cap_sr[91],stream_cap_sr[90],1'b1,stream_cap_sr[90],
                            stream_cap_sr[80],1'b1,stream_cap_sr[80],stream_cap_sr[81],1'b1,stream_cap_sr[81],
                            stream_cap_sr[83],1'b1,stream_cap_sr[83],stream_cap_sr[82],1'b1,stream_cap_sr[82],
                            stream_cap_sr[104],1'b1,stream_cap_sr[104],stream_cap_sr[105],1'b1,stream_cap_sr[105],
                            stream_cap_sr[107],1'b1,stream_cap_sr[107],stream_cap_sr[106],1'b1,stream_cap_sr[106],
                            stream_cap_sr[96],1'b1,stream_cap_sr[96],stream_cap_sr[97],1'b1,stream_cap_sr[97],
                            stream_cap_sr[99],1'b1,stream_cap_sr[99],stream_cap_sr[98],1'b1,stream_cap_sr[98],
                            stream_cap_sr[124],1'b1,stream_cap_sr[124],stream_cap_sr[125],1'b1,stream_cap_sr[125],
                            stream_cap_sr[127],1'b1,stream_cap_sr[127],stream_cap_sr[126],1'b1,stream_cap_sr[126],
                            stream_cap_sr[116],1'b1,stream_cap_sr[116],stream_cap_sr[117],1'b1,stream_cap_sr[117],
                            stream_cap_sr[119],1'b1,stream_cap_sr[119],stream_cap_sr[118],1'b1,stream_cap_sr[118]};

               end
             endcase
          end
        end
        CLK_NEG: begin // Load the Shift register
          sr_state  <= CLK_POS;
          LED_CLK0  <= !LED_CLK0;
          led_sr    <= {1'b0,led_sr[191:1]};
          LED_DATA0 <= led_sr[0];
        end
        CLK_POS: begin // Simply move to CLK_NEG state
          LED_CLK0       <= !LED_CLK0;
          led_sr_counter <= led_sr_counter + 'b1;
          if (led_sr_counter == 8'd192) begin
            sr_state <= IDLE;
          end
          else begin
            sr_state <= CLK_NEG;
          end
        end
      endcase
    end
  end

  always @(posedge clk)begin
    if (rst == 1'b0)begin
      LED_CLEAR0        <= 1'b0;
      LED_CLK0_BUF_EN   <= 1'b0;
      end
      else if(stream_cap_done)begin
        LED_CLEAR0        <= 1'b1;
        LED_CLK0_BUF_EN   <= 1'b1;
      end
      else if (led_sr_counter == 8'd192)begin
        LED_CLEAR0        <= 1'b0;
        LED_CLK0_BUF_EN   <= 1'b0;
      end
  end

endmodule