/*

reg 0x00 ~ 0x1F are read only register
reg 0x20 ~ 0x3F are r/w registers

*/
//-----------------------------------------------------------------------------
// Title       : Register file module
// Design      : Wedge100 CPLD Design
// File        : reg_file.v
//-----------------------------------------------------------------------------
// Version     : Ver 2.0
// Changes     : 1. Added 3 read/write register rw_0x41, rw_0x44,rw_0x45 and 3 read only 
//               register ro_0x40,ro_0x42,ro_0x43 for additional function implementation
//-----------------------------------------------------------------------------
// Version     : Ver 2.1
// Changes     : 1. Added a write register rw_0x46[1:0] to select the LED implementation speed
//-----------------------------------------------------------------------------
// Description:- Register description and implementation module
//-----------------------------------------------------------------------------


module reg_file #
( 
    parameter        DUAL_BOOT_DEFAULT  = 1'b0   // in default disable dual boot 
)
(
    input            clk,
    input            rst_pwr_n,         
    
    input            hot_reset_n,
    input            warm_reset_n,
    input            cold_reset_n, 
   
    output  reg      wr_0x31_en_buf,
 
    input            read_from_bmc,
    input            write_from_bmc,
    input  [7:0]     address_from_bmc,
    input  [7:0]     writedata_from_bmc,
    
    output [7:0]     readdata_to_bmc_usrv,
    output [7:0]     readdata_nodelay, //read data without sclk delay

    input            read_from_usrv,
    input            write_from_usrv,
    input  [7:0]     address_from_usrv,
    input  [7:0]     writedata_from_usrv,
 

    input  [7:0]     ro_0x00,
    input  [7:0]     ro_0x01,
    input  [7:0]     ro_0x02,
    input  [7:0]     ro_0x03,
    input  [7:0]     ro_0x04,
    input  [7:0]     ro_0x05,
    input  [7:0]     ro_0x06,
    input  [7:0]     ro_0x07,
    input  [7:0]     ro_0x08,
    input  [7:0]     ro_0x09,
    input  [7:0]     ro_0x0a,
    input  [7:0]     ro_0x0b,
    input  [7:0]     ro_0x0c,
    input  [7:0]     ro_0x0d,
    input  [7:0]     ro_0x0e, 
    input  [7:0]     ro_0x0f,
    
    input  [7:0]     ro_0x10,  // psu status
    input  [7:0]     ro_0x11,  // on board power status register 1
    input  [7:0]     ro_0x12,  // on board power status register 2
    input  [7:0]     ro_0x13,  // pca9535 INT_N status
    input  [7:0]     ro_0x14,
    input  [7:0]     ro_0x15,
    input  [7:0]     ro_0x16,  // rov status rgister
    input  [7:0]     ro_0x17,
    input  [7:0]     ro_0x18,
    input  [7:0]     ro_0x19,
    input  [7:0]     ro_0x1a,
    input  [7:0]     ro_0x1b,
    input  [7:0]     ro_0x1c,
    input  [7:0]     ro_0x1d,
    input  [7:0]     ro_0x1e, 
    input  [7:0]     ro_0x1f,  // cp2112 input register      

    input [7:0]      ro_0x40, // GPIO read register     
    input [7:0]      ro_0x42, // GPIO read register     
    input [7:0]      ro_0x43, // GPIO read register     
	
    output reg [7:0] rw_0x20,  // psu interrupt block register
    output reg [7:0] rw_0x21,  // pwr_mb interrupt block registers
    output reg [7:0] rw_0x22,  // pwr_ir interrupt block registers
    output reg [7:0] rw_0x23,  //  pca9535 interrupt block register 
    output reg [7:0] rw_0x24,  // blobal Interrupt block register 
    output reg [7:0] rw_0x25,  // uart mux control
    output reg [7:0] rw_0x26,
    output reg [7:0] rw_0x27,
    output reg [7:0] rw_0x28,
    output reg [7:0] rw_0x29,
    output reg [7:0] rw_0x2a,
    output reg [7:0] rw_0x2b,
    output reg [7:0] rw_0x2c,
    output reg [7:0] rw_0x2d, 
    output reg [7:0] rw_0x2e,  
    output reg [7:0] rw_0x2f,  
    
    output reg [7:0] rw_0x30,
    output reg [7:0] rw_0x31,
    output reg [7:0] rw_0x32,
    output reg [7:0] rw_0x33,
    output reg [7:0] rw_0x34,
    output reg [7:0] rw_0x35,
    output reg [7:0] rw_0x36,
    output reg [7:0] rw_0x37,
    output reg [7:0] rw_0x38,
    output reg [7:0] rw_0x39,
    output reg [7:0] rw_0x3a,
    output reg [7:0] rw_0x3b,
    output reg [7:0] rw_0x3c,  // led control for tomahawk 1
    output reg [7:0] rw_0x3d,  // led control for tomahawk 2
    output reg [7:0] rw_0x3e,  // led control for system led 1
    output reg [7:0] rw_0x3f,  // led control for system led 2

    output reg [7:0] rw_0x41, // GPIO read register     
    output reg [7:0] rw_0x44,  // misc interrupt write register and clock pll reset register  
    output reg [7:0] rw_0x45,   // GPIO write register  
    output reg [7:0] rw_0x46   // LED speed selection register  
   
) ;


wire       power_on_reset_n = rst_pwr_n;      // poweron reset from power1014
wire [7:0] cpu_rd_dat;
                                
    
       	 
//---------------------------------------------------------------------
//      CPU Bus for I2C BMC and usrv
//---------------------------------------------------------------------
    wire       cpu_read, cpu_write;
    wire [7:0] cpu_address, cpu_writedata;
    reg  [7:0] cpu_readdata;

//    assign  cpld_busy = 1'b0;

    wire [7:0] addr_bmc  = ( read_from_bmc  || write_from_bmc )? address_from_bmc :8'd0;
    wire [7:0] addr_usrv = ( read_from_usrv || write_from_usrv)? address_from_usrv:8'd0;
    
    wire [7:0] wr_data_bmc  = ( write_from_bmc )? writedata_from_bmc :8'd0;
    wire [7:0] wr_data_usrv = ( write_from_usrv)? writedata_from_usrv:8'd0;
     
    //assign  cpu_address         = (address_from_bmc | address_from_usrv);
    assign  cpu_address         = (addr_bmc | addr_usrv);
    
    assign  readdata_to_bmc_usrv = cpu_readdata;
    //assign  cpu_writedata       = ( writedata_from_i2c | writedata_from_spi );
    assign  cpu_writedata       = ( wr_data_bmc | wr_data_usrv );
    
    assign  cpu_read            = ( read_from_bmc  | read_from_usrv );
    assign  cpu_write           = ( write_from_bmc | write_from_usrv);

    wire rd_0x00_en  = ( cpu_read & ( cpu_address == 8'h00) );
    wire rd_0x01_en  = ( cpu_read & ( cpu_address == 8'h01) );
    wire rd_0x02_en  = ( cpu_read & ( cpu_address == 8'h02) );
    wire rd_0x03_en  = ( cpu_read & ( cpu_address == 8'h03) );
    wire rd_0x04_en  = ( cpu_read & ( cpu_address == 8'h04) );
    wire rd_0x05_en  = ( cpu_read & ( cpu_address == 8'h05) );
    wire rd_0x06_en  = ( cpu_read & ( cpu_address == 8'h06) );
    wire rd_0x07_en  = ( cpu_read & ( cpu_address == 8'h07) );
    wire rd_0x08_en  = ( cpu_read & ( cpu_address == 8'h08) );
    wire rd_0x09_en  = ( cpu_read & ( cpu_address == 8'h09) );
    wire rd_0x0a_en  = ( cpu_read & ( cpu_address == 8'h0a) );
    wire rd_0x0b_en  = ( cpu_read & ( cpu_address == 8'h0b) );
    wire rd_0x0c_en  = ( cpu_read & ( cpu_address == 8'h0c) );
    wire rd_0x0d_en  = ( cpu_read & ( cpu_address == 8'h0d) );
    wire rd_0x0e_en  = ( cpu_read & ( cpu_address == 8'h0e) );
    wire rd_0x0f_en  = ( cpu_read & ( cpu_address == 8'h0f) );
    
    wire rd_0x10_en  = ( cpu_read & ( cpu_address == 8'h10) );
    wire rd_0x11_en  = ( cpu_read & ( cpu_address == 8'h11) );
    wire rd_0x12_en  = ( cpu_read & ( cpu_address == 8'h12) );
    wire rd_0x13_en  = ( cpu_read & ( cpu_address == 8'h13) );
    wire rd_0x14_en  = ( cpu_read & ( cpu_address == 8'h14) );
    wire rd_0x15_en  = ( cpu_read & ( cpu_address == 8'h15) );
    wire rd_0x16_en  = ( cpu_read & ( cpu_address == 8'h16) );
    wire rd_0x17_en  = ( cpu_read & ( cpu_address == 8'h17) );
    wire rd_0x18_en  = ( cpu_read & ( cpu_address == 8'h18) );
    wire rd_0x19_en  = ( cpu_read & ( cpu_address == 8'h19) );
    wire rd_0x1a_en  = ( cpu_read & ( cpu_address == 8'h1a) );
    wire rd_0x1b_en  = ( cpu_read & ( cpu_address == 8'h1b) );
    wire rd_0x1c_en  = ( cpu_read & ( cpu_address == 8'h1c) );
    wire rd_0x1d_en  = ( cpu_read & ( cpu_address == 8'h1d) );
    wire rd_0x1e_en  = ( cpu_read & ( cpu_address == 8'h1e) );
    wire rd_0x1f_en  = ( cpu_read & ( cpu_address == 8'h1f) );
        
    wire rd_0x20_en  = ( cpu_read & ( cpu_address == 8'h20) );
    wire rd_0x21_en  = ( cpu_read & ( cpu_address == 8'h21) );
    wire rd_0x22_en  = ( cpu_read & ( cpu_address == 8'h22) );
    wire rd_0x23_en  = ( cpu_read & ( cpu_address == 8'h23) );
    wire rd_0x24_en  = ( cpu_read & ( cpu_address == 8'h24) );
    wire rd_0x25_en  = ( cpu_read & ( cpu_address == 8'h25) );
    wire rd_0x26_en  = ( cpu_read & ( cpu_address == 8'h26) );
    wire rd_0x27_en  = ( cpu_read & ( cpu_address == 8'h27) );
    wire rd_0x28_en  = ( cpu_read & ( cpu_address == 8'h28) );
    wire rd_0x29_en  = ( cpu_read & ( cpu_address == 8'h29) );
    wire rd_0x2a_en  = ( cpu_read & ( cpu_address == 8'h2a) );
    wire rd_0x2b_en  = ( cpu_read & ( cpu_address == 8'h2b) );
    wire rd_0x2c_en  = ( cpu_read & ( cpu_address == 8'h2c) );
    wire rd_0x2d_en  = ( cpu_read & ( cpu_address == 8'h2d) );
    wire rd_0x2e_en  = ( cpu_read & ( cpu_address == 8'h2e) );
    wire rd_0x2f_en  = ( cpu_read & ( cpu_address == 8'h2f) );
    
    wire rd_0x30_en  = ( cpu_read & ( cpu_address == 8'h30) );
    wire rd_0x31_en  = ( cpu_read & ( cpu_address == 8'h31) );
    wire rd_0x32_en  = ( cpu_read & ( cpu_address == 8'h32) );
    wire rd_0x33_en  = ( cpu_read & ( cpu_address == 8'h33) );
    wire rd_0x34_en  = ( cpu_read & ( cpu_address == 8'h34) );
    wire rd_0x35_en  = ( cpu_read & ( cpu_address == 8'h35) );
    wire rd_0x36_en  = ( cpu_read & ( cpu_address == 8'h36) );
    wire rd_0x37_en  = ( cpu_read & ( cpu_address == 8'h37) );
    wire rd_0x38_en  = ( cpu_read & ( cpu_address == 8'h38) );
    wire rd_0x39_en  = ( cpu_read & ( cpu_address == 8'h39) );
    wire rd_0x3a_en  = ( cpu_read & ( cpu_address == 8'h3a) );
    wire rd_0x3b_en  = ( cpu_read & ( cpu_address == 8'h3b) );
    wire rd_0x3c_en  = ( cpu_read & ( cpu_address == 8'h3c) );
    wire rd_0x3d_en  = ( cpu_read & ( cpu_address == 8'h3d) );
    wire rd_0x3e_en  = ( cpu_read & ( cpu_address == 8'h3e) );
    wire rd_0x3f_en  = ( cpu_read & ( cpu_address == 8'h3f) );
    
   
    wire wr_0x20_en  = ( cpu_write & ( cpu_address == 8'h20) );
    wire wr_0x21_en  = ( cpu_write & ( cpu_address == 8'h21) );
    wire wr_0x22_en  = ( cpu_write & ( cpu_address == 8'h22) );
    wire wr_0x23_en  = ( cpu_write & ( cpu_address == 8'h23) );
    wire wr_0x24_en  = ( cpu_write & ( cpu_address == 8'h24) );
    wire wr_0x25_en  = ( cpu_write & ( cpu_address == 8'h25) );
    wire wr_0x26_en  = ( cpu_write & ( cpu_address == 8'h26) );
    wire wr_0x27_en  = ( cpu_write & ( cpu_address == 8'h27) );
    wire wr_0x28_en  = ( cpu_write & ( cpu_address == 8'h28) );
    wire wr_0x29_en  = ( cpu_write & ( cpu_address == 8'h29) );
    wire wr_0x2a_en  = ( cpu_write & ( cpu_address == 8'h2a) );
    wire wr_0x2b_en  = ( cpu_write & ( cpu_address == 8'h2b) );
    wire wr_0x2c_en  = ( cpu_write & ( cpu_address == 8'h2c) );
    wire wr_0x2d_en  = ( cpu_write & ( cpu_address == 8'h2d) );
    wire wr_0x2e_en  = ( cpu_write & ( cpu_address == 8'h2e) );
    wire wr_0x2f_en  = ( cpu_write & ( cpu_address == 8'h2f) );  
   
    wire wr_0x30_en  = ( cpu_write & ( cpu_address == 8'h30) );
    wire wr_0x31_en  = ( cpu_write & ( cpu_address == 8'h31) );
    wire wr_0x32_en  = ( cpu_write & ( cpu_address == 8'h32) );
    wire wr_0x33_en  = ( cpu_write & ( cpu_address == 8'h33) );
    wire wr_0x34_en  = ( cpu_write & ( cpu_address == 8'h34) );
    wire wr_0x35_en  = ( cpu_write & ( cpu_address == 8'h35) );
    wire wr_0x36_en  = ( cpu_write & ( cpu_address == 8'h36) );
    wire wr_0x37_en  = ( cpu_write & ( cpu_address == 8'h37) );
    wire wr_0x38_en  = ( cpu_write & ( cpu_address == 8'h38) );
    wire wr_0x39_en  = ( cpu_write & ( cpu_address == 8'h39) );
    wire wr_0x3a_en  = ( cpu_write & ( cpu_address == 8'h3a) );
    wire wr_0x3b_en  = ( cpu_write & ( cpu_address == 8'h3b) );
    wire wr_0x3c_en  = ( cpu_write & ( cpu_address == 8'h3c) );
    wire wr_0x3d_en  = ( cpu_write & ( cpu_address == 8'h3d) );
    wire wr_0x3e_en  = ( cpu_write & ( cpu_address == 8'h3e) );
    wire wr_0x3f_en  = ( cpu_write & ( cpu_address == 8'h3f) );
    
    wire rd_0x40_en  = ( cpu_read & ( cpu_address == 8'h40) );
    wire rd_0x42_en  = ( cpu_read & ( cpu_address == 8'h42) );
    wire rd_0x43_en  = ( cpu_read & ( cpu_address == 8'h43) );
    wire rd_0x41_en  = ( cpu_read & ( cpu_address == 8'h41) );
    wire rd_0x44_en  = ( cpu_read & ( cpu_address == 8'h44) );
    wire rd_0x45_en  = ( cpu_read & ( cpu_address == 8'h45) );
    wire rd_0x46_en  = ( cpu_read & ( cpu_address == 8'h46) );
            
    wire wr_0x41_en  = ( cpu_write & ( cpu_address == 8'h41) );
    wire wr_0x44_en  = ( cpu_write & ( cpu_address == 8'h44) );
    wire wr_0x45_en  = ( cpu_write & ( cpu_address == 8'h45) );
    wire wr_0x46_en  = ( cpu_write & ( cpu_address == 8'h46) );
  
    
    
//---------------------------------------------------------------------
//  CPU Read Data via I2C or SPI
//---------------------------------------------------------------------
assign  readdata_nodelay = cpu_rd_dat; 
assign  cpu_rd_dat   = rd_0x00_en ? ro_0x00:    // Register 0x00:  
                       rd_0x01_en ? ro_0x01:    // Register 0x01:  
                       rd_0x02_en ? ro_0x02:    // Register 0x02:  
                       rd_0x03_en ? ro_0x03:    // Register 0x03:  
                       rd_0x04_en ? ro_0x04:    // Register 0x04:  
                       rd_0x05_en ? ro_0x05:    // Register 0x05:  
                       rd_0x06_en ? ro_0x06:    // Register 0x06:  
                       rd_0x07_en ? ro_0x07:    // Register 0x07:  
                       rd_0x08_en ? ro_0x08:    // Register 0x08:  
                       rd_0x09_en ? ro_0x09:    // Register 0x09:  
                       rd_0x0a_en ? ro_0x0a:    //                  
                       rd_0x0b_en ? ro_0x0b:    // Register 0x0b: ROV status register                 
                       rd_0x0c_en ? ro_0x0c:    //                  
                       rd_0x0d_en ? ro_0x0d:    //                  
                       rd_0x0e_en ? ro_0x0e:    // Register 0x0e: reset reason register            
                       rd_0x0f_en ? ro_0x0f:    // Register 0x0f: reset reason status bit register 
                       //
                       rd_0x10_en ? ro_0x10:    // Register 0x10: psu status register
                       rd_0x11_en ? ro_0x11:    // Register 0x11: power status
                       rd_0x12_en ? ro_0x12:    // Register 0x12: power status
                       rd_0x13_en ? ro_0x13:    // Register 0x13: I2C pca9535 interrupt status
                       rd_0x14_en ? ro_0x14:    // Register 0x14: reserved for interrupt status, write 1 to clear register
                       rd_0x15_en ? ro_0x15:    // Register 0x15: reserved for interrupt status, write 1 to clear register 
                       rd_0x16_en ? ro_0x16:    // Register 0x16: reserved for interrupt status, write 1 to clear register
                       rd_0x17_en ? ro_0x17:    // Register 0x17: reserved for interrupt status, write 1 to clear register
                       rd_0x18_en ? ro_0x18:    // 
                       rd_0x19_en ? ro_0x19:    // 
                       rd_0x1a_en ? ro_0x1a:    // 
                       rd_0x1b_en ? ro_0x1b:    // 
                       rd_0x1c_en ? ro_0x1c:    // 
                       rd_0x1d_en ? ro_0x1d:    // 
                       rd_0x1e_en ? ro_0x1e:    // 
                       rd_0x1f_en ? ro_0x1f:    // 
                                             
                       rd_0x20_en ? rw_0x20:    // Register 0x20: interrupt block register: psu
                       rd_0x21_en ? rw_0x21:    // Register 0x21: interrupt block register: main board power
                       rd_0x22_en ? rw_0x22:    // Register 0x22: interrupt block register: IR power 
                       rd_0x23_en ? rw_0x23:    // Register 0x23: interrupt block register: PCA9535 interrupt
                       rd_0x24_en ? rw_0x24:    // Register 0x24: interrupt block register: global interrupt block
                       rd_0x25_en ? rw_0x25:    // Register 0x25
                       rd_0x26_en ? rw_0x26:    // Register 0x26 
                       rd_0x27_en ? rw_0x27:    // Register 0x27
                       rd_0x28_en ? rw_0x28:    // Register 0x28
                       rd_0x29_en ? rw_0x29:    // Register 0x29
                       rd_0x2a_en ? rw_0x2a:    // Register 0x2a
                       rd_0x2b_en ? rw_0x2b:    // Register 0x2b
                       rd_0x2c_en ? rw_0x2c:    // Register 0x2c
                       rd_0x2d_en ? rw_0x2d:    // Register 0x2d
                       rd_0x2e_en ? rw_0x2e:    // Register 0x2e
                       rd_0x2f_en ? rw_0x2f:    // Register 0x2f
                       
                       rd_0x30_en ? rw_0x30:    // Register 0x30  
                       rd_0x31_en ? rw_0x31:    // Register 0x31  
                       rd_0x32_en ? rw_0x32:    // Register 0x32  
                       rd_0x33_en ? rw_0x33:    // Register 0x33  
                       rd_0x34_en ? rw_0x34:    // Register 0x34  
                       rd_0x35_en ? rw_0x35:    // Register 0x35  
                       rd_0x36_en ? rw_0x36:    // Register 0x36  
                       rd_0x37_en ? rw_0x37:    // Register 0x37  
                       rd_0x38_en ? rw_0x38:    // Register 0x38  
                       rd_0x39_en ? rw_0x39:    // Register 0x39  
                       rd_0x3a_en ? rw_0x3a:    // Register 0x3a : BMC reset enable register 
                       rd_0x3b_en ? rw_0x3b:    // Register 0x3b : general reset enable register 
                       rd_0x3c_en ? rw_0x3c:    // Register 0x3c  
                       rd_0x3d_en ? rw_0x3d:    // Register 0x3d  
                       rd_0x3e_en ? rw_0x3e:    // Register 0x3e  
                       rd_0x3f_en ? rw_0x3f:    // Register 0x3f  
					   
					   rd_0x40_en ? ro_0x40:
					   rd_0x41_en ? rw_0x41:
					   rd_0x42_en ? ro_0x42:
					   rd_0x43_en ? ro_0x43:
					   rd_0x44_en ? rw_0x44:
					   rd_0x45_en ? rw_0x45:
					   rd_0x46_en ? rw_0x46:
					   
                                      8'd0;
  
 
always @ (posedge clk  ) begin
    wr_0x31_en_buf	<= wr_0x31_en; 
end

                                        

always @ (posedge clk, negedge power_on_reset_n )                           
if ( !power_on_reset_n )        cpu_readdata <= 8'd0 ; 
else     cpu_readdata <= cpu_rd_dat;
                  
  

//---------------------------------------------------------------------
//  CPU Write Data via I2C  
//---------------------------------------------------------------------

   // Register 0x20 -   (R/W)
    always @ (posedge clk, negedge warm_reset_n ) begin 
        if ( !warm_reset_n )            rw_0x20 <= 8'b0000_0000 ; 
        else if ( wr_0x20_en )          rw_0x20 <= cpu_writedata ;
    end  

  // Register 0x21 -   (R/W)
    always @ (posedge clk, negedge warm_reset_n ) begin 
        if ( !warm_reset_n )            rw_0x21 <= 8'b0000_0000 ; 
        else if ( wr_0x21_en )          rw_0x21 <= cpu_writedata ;
    end  

   // Register 0x22 -   (R/W)
    always @ (posedge clk, negedge warm_reset_n ) begin 
        if ( !warm_reset_n )            rw_0x22 <= 8'b0000_0000 ; 
        else if ( wr_0x22_en )          rw_0x22 <= cpu_writedata ;
    end  

  // Register 0x23 -   (R/W)
    always @ (posedge clk, negedge warm_reset_n ) begin 
        if ( !warm_reset_n )            rw_0x23 <= 8'b0000_0000 ; 
        else if ( wr_0x23_en )          rw_0x23 <= cpu_writedata ;
    end  

   // Register 0x24 -   (R/W)
    always @ (posedge clk, negedge warm_reset_n ) begin 
        if ( !warm_reset_n )            rw_0x24 <= 8'b0000_0000 ; 
        else if ( wr_0x24_en )          rw_0x24 <= cpu_writedata ;
    end  

  // Register 0x25 -   (R/W)
    always @ (posedge clk, negedge warm_reset_n ) begin 
        if ( !warm_reset_n )            rw_0x25 <= 8'b0000_0000 ; 
        else if ( wr_0x25_en )          rw_0x25 <= cpu_writedata ;
    end  

   // Register 0x26 -   (R/W)
    always @ (posedge clk, negedge warm_reset_n ) begin 
        if ( !warm_reset_n )            rw_0x26 <= 8'b0000_0000 ;   // default set to 0000_0000 to connect panther+ console to BMC uart-1 (FB debug console)
        else if ( wr_0x26_en )          rw_0x26 <= cpu_writedata ;
    end  

  // Register 0x27 -   (R/W)
    always @ (posedge clk, negedge warm_reset_n ) begin 
        if ( !warm_reset_n )            rw_0x27 <= 8'b0000_0000 ; 
        else if ( wr_0x27_en )          rw_0x27 <= cpu_writedata ;
    end  

   // Register 0x28 -   (R/W)
    always @ (posedge clk, negedge warm_reset_n ) begin 
        if ( !warm_reset_n )            rw_0x28 <= 8'b1100_1111 ; // change pwrok default value to 1
        else if ( wr_0x28_en )          rw_0x28 <= cpu_writedata ;
    end  

  // Register 0x29 -   (R/W)
    always @ (posedge clk, negedge warm_reset_n ) begin 
        if ( !warm_reset_n )            rw_0x29 <= 8'b0000_0000 ; 
        else if ( wr_0x29_en )          rw_0x29 <= cpu_writedata ;
    end  

   // Register 0x2a -   (R/W)
    always @ (posedge clk, negedge warm_reset_n ) begin 
        if ( !warm_reset_n )            rw_0x2a <= 8'b0000_0000 ; 
        else if ( wr_0x2a_en )          rw_0x2a <= cpu_writedata ;
    end  

  // Register 0x2b -   (R/W)
    always @ (posedge clk, negedge warm_reset_n ) begin 
        if ( !warm_reset_n )            rw_0x2b <= 8'b0000_0000 ; 
        else if ( wr_0x2b_en )          rw_0x2b <= cpu_writedata ;
    end  

   // Register 0x2c -   (R/W)
    always @ (posedge clk, negedge warm_reset_n ) begin 
        if ( !warm_reset_n )            rw_0x2c <= 8'b0000_0000 ; 
        else if ( wr_0x2c_en )          rw_0x2c <= cpu_writedata ;
    end  

  // Register 0x2d -   (R/W)
    always @ (posedge clk, negedge warm_reset_n ) begin 
        if ( !warm_reset_n )            rw_0x2d <= 8'b0000_0000 ;    
        else if ( wr_0x2d_en )          rw_0x2d <= cpu_writedata ;
    end  

   // Register 0x2e -   (R/W)
    always @ (posedge clk, negedge power_on_reset_n  ) begin 
     // if ( !power_on_reset_n )        rw_0x2e <= 8'b1000_1000 ; // disable dynamic ROV control.  then default value is 1.0V(VIDSEL = 100)
        if ( !power_on_reset_n )        rw_0x2e <= 8'b1001_1000 ; // enable dynamic ROV control. if pd_main_reset_n = 0, then default value is 1.0V(VIDSEL = 100) ; enable heart-attack
    //   if ( !power_on_reset_n )        rw_0x2e <= 8'b0000_0000 ; 
        else if ( wr_0x2e_en )          rw_0x2e <= cpu_writedata ;
    end  

  // Register 0x2f -   (R/W)
    always @ (posedge clk, negedge power_on_reset_n ) begin 
        //if ( !warm_reset_n )            rw_0x2f <= 8'b0000_0000 ; 
        // if ( !power_on_reset_n )        rw_0x2f <= 8'b1111_1011 ;    // force mode to enable buffer when main power is OK       => dal_boot enable, and second SPI can be write
        //if ( !power_on_reset_n )        rw_0x2f <= {7'b1111_101, DUAL_BOOT_DEFAULT } ;
        if ( !power_on_reset_n )        rw_0x2f <= {7'b1111_001, DUAL_BOOT_DEFAULT } ;  // allow BMC to control micro-server power button
        else if ( wr_0x2f_en )          rw_0x2f <= cpu_writedata ;
    end  

    
   // Register 0x30 -   (R/W)
    always @ (posedge clk, negedge power_on_reset_n ) begin 
        //if ( !power_on_reset_n )        rw_0x30 <= 8'b1111_1111 ;   // enable main and microserver power
        if ( !power_on_reset_n )        rw_0x30 <= 8'b1111_0001 ;  // disable main and micro-server power in default
        else if ( wr_0x30_en )          rw_0x30 <= cpu_writedata ;
    end  

    
    // Register 0x31 - Reset Control 2 (R/W)
    // 
    always @ (posedge clk, negedge cold_reset_n ) begin
        if ( !cold_reset_n )            rw_0x31 <= 8'b1111_1111 ;
        //else if ( wr_0x31_en && reset_protect_n)          rw_0x31 <= cpu_writedata ;
        else if ( wr_0x31_en )          rw_0x31 <= cpu_writedata ;
        // some reset bits are pulse, not latched signal
        else                            rw_0x31 <= {rw_0x31[7],7'b1111111} ;  
    end

   

   // Register 0x32 -   (R/W)
    always @ (posedge clk, negedge warm_reset_n ) begin 
        if ( !warm_reset_n )            rw_0x32 <= 8'b0000_1111 ; 
        else if ( wr_0x32_en )          rw_0x32 <= cpu_writedata ;
    end  

  // Register 0x33 -   (R/W)
    always @ (posedge clk, negedge warm_reset_n ) begin 
        //if ( !warm_reset_n )            rw_0x33 <= 8'b0000_0000 ; 
        //if ( !warm_reset_n )            rw_0x33 <= 8'b0000_0100 ;  // enable uart isolation buffer in default
        if ( !warm_reset_n )            rw_0x33 <= 8'b0000_0110 ; // rev 3.51 change default value to 0000_0110
        else if ( wr_0x33_en )          rw_0x33 <= cpu_writedata ;
    end  

   // Register 0x34 -   (R/W)
    always @ (posedge clk, negedge warm_reset_n ) begin 
        if ( !warm_reset_n )            rw_0x34 <= 8'b1111_1111 ; 
        else if ( wr_0x34_en )          rw_0x34 <= cpu_writedata ;
    end  

  // Register 0x35 -   (R/W)
    always @ (posedge clk, negedge warm_reset_n ) begin 
        if ( !warm_reset_n )            rw_0x35 <= 8'b1111_1111 ; 
        else if ( wr_0x35_en )          rw_0x35 <= cpu_writedata ;
    end  

   // Register 0x36 -   (R/W)
    always @ (posedge clk, negedge warm_reset_n ) begin 
        if ( !warm_reset_n )            rw_0x36 <= 8'b1111_1111 ; 
        else if ( wr_0x36_en )          rw_0x36 <= cpu_writedata ;
    end  

  // Register 0x37 -   (R/W)
    always @ (posedge clk, negedge warm_reset_n ) begin 
        if ( !warm_reset_n )            rw_0x37 <= 8'b1111_1111 ; 
        else if ( wr_0x37_en )          rw_0x37 <= cpu_writedata ;
    end    
    
    // Register 0x38 
    always @ (posedge clk, negedge warm_reset_n ) begin
        if ( !warm_reset_n )            rw_0x38 <= 8'b1111_1111 ;
        else if ( wr_0x38_en )          rw_0x38 <= cpu_writedata ;
     end
    // Register 0x39  
    always @ (posedge clk, negedge warm_reset_n ) begin
        //if ( !warm_reset_n )            rw_0x39 <= 8'b0000_1010 ;
        if ( !warm_reset_n )            rw_0x39 <= 8'b1111_1111 ; // led off in default
        else if ( wr_0x39_en )          rw_0x39 <= cpu_writedata ;
     end
    // Register 0x3a  , need poweron reset because it control BMC reset enable and dsiable
    // always @ (posedge clk, negedge warm_reset_n ) begin
    //    if ( !warm_reset_n )            rw_0x3a <= 8'b1111_0000 ;
     always @ (posedge clk, negedge power_on_reset_n ) begin      
        //if ( !power_on_reset_n )        rw_0x3a <= 8'b1111_1111 ;    
        if ( !power_on_reset_n )        rw_0x3a <= 8'b0000_0000 ; 
        else if ( wr_0x3a_en )          rw_0x3a <= cpu_writedata ;
    end
    // Register 0x3b -  
    always @ (posedge clk, negedge power_on_reset_n ) begin
        //if ( !power_on_reset_n )        rw_0x3b <= 8'b0000_1111 ; 
        if ( !power_on_reset_n )        rw_0x3b <= 8'b0000_1111 ;     // in default enable front port reset button
        else if ( wr_0x3b_en )          rw_0x3b    <= cpu_writedata ;
     end
    // Register 0x3c - 
    always @ (posedge clk, negedge warm_reset_n ) begin
        //if ( !warm_reset_n )            rw_0x3c <= 8'b1001_1000 ;    // default enable led walking test
        if ( !warm_reset_n )            rw_0x3c <= 8'b1110_0000 ;    // default enable led pride test
        else if ( wr_0x3c_en )          rw_0x3c <= cpu_writedata ;
     end
    // Register 0x3d -  
    always @ (posedge clk, negedge warm_reset_n ) begin
        if ( !warm_reset_n )            rw_0x3d <= 8'b0100_0000 ;   // blue in default for led test
        else if ( wr_0x3d_en )          rw_0x3d <= cpu_writedata ;
    end
    // Register 0x3e  
    always @ (posedge clk, negedge warm_reset_n ) begin
        if ( !warm_reset_n )            rw_0x3e <= 8'b0000_1100 ; // blue blink
        else if ( wr_0x3e_en )          rw_0x3e <= cpu_writedata ;
     end
    // Register 0x3f  
    always @ (posedge clk, negedge warm_reset_n ) begin
        if ( !warm_reset_n )            rw_0x3f <= 8'b0000_0010 ;   // constant green
        else if ( wr_0x3f_en )          rw_0x3f <= cpu_writedata ;
    end
 
   // Register 0x41 -   (R/W)
    always @ (posedge clk, negedge warm_reset_n ) begin 
        if ( !warm_reset_n )            rw_0x41 <= 8'b1111_1111 ;  // MISC interrupt enable register
        else if ( wr_0x41_en )          rw_0x41 <= cpu_writedata ;
    end  
  
   // Register 0x44 -   (R/W) 
    always @ (posedge clk, negedge warm_reset_n ) begin 
        if ( !warm_reset_n )            rw_0x44 <= 8'b0000_0111 ; // Xpliant CLK_PLL_RESET register
        else if ( wr_0x44_en )          rw_0x44 <= cpu_writedata ;
    end  

 
 // Register 0x45 -   (R/W)
    always @ (posedge clk, negedge warm_reset_n ) begin 
        if ( !warm_reset_n )            rw_0x45 <= 8'b0000_0000 ;  // GPIO write register
        else if ( wr_0x45_en )          rw_0x45 <= cpu_writedata ;
    end  
  
   // Register 0x46 -   (R/W)
    always @ (posedge clk, negedge warm_reset_n ) begin 
        if ( !warm_reset_n )            rw_0x46 <= 8'b0000_0000 ;  // GPIO write register
        else if ( wr_0x46_en )          rw_0x46 <= cpu_writedata ;
    end 
     
    
                                                                      
endmodule