// This module is to multiplex reset source and detect reset event to trigger reset output
//
 
//  
// reg 0x30 reset control register bit definition
// bit7 resetpwr_req   : this bit will drive power607 to shutdown power. this is real power cycling on board. active low
// bit6 reset_cold_req : write 0 to this bit will cause warm boot for all devices including cpu itself
// bit5 reset_warm_req : write 0 to this bit will cause cold boot for all devices including cpu itself
// bit4 reset_hot_req : write 0 to this bit will cause warm boot for cpu system only including NOR flash, but not MAC and other devices
// bit3 reserved       : reserved
// bit2  
// bit1 reserved
// bit0 reserved
//-----------------------------------------------------------------------------
// Title       : reset trigger module
// Design      : Wedge100 CPLD Design
// File        : reset_trigger.v
//-----------------------------------------------------------------------------
// Version     : Ver 2.0
// Changes     : 1. Renamed the signals mentioned in excel sheet
//-----------------------------------------------------------------------------
// Description:- Reset trigger implementation
//-----------------------------------------------------------------------------

module reset_trigger 
(
  input		   clk					       // clock 

, input        reg_rst_ctrl_wr             // write access to reg 0x31, used to generate pulse, active high                
, input [7:0]  reg_rst_ctrl                // reset control register, 0x31 
, input [7:0]  reg_bmc_rst_en              // BMC reset request enable register,         rw_0x3a
, input [2:0]  reg_gen_rst_en              // general reset request enable register,     rw_0x3b                                                                                                   
                                       
//, input        MICROSRV_PCIE_RST_N       // micro-server reset output               (* chip_pin = "E7"  *)  : Microserver request to reset PCIe devices   
                                                                                                    
, input	       LS_BMC_WDTRST1                 // BMC watch dog timer reset output,       (* chip_pin = "B5"  *)   falling edge pulse, watchdog timeout, reset BMC only and reboot to secondary image
, input	       LS_BMC_WDTRST2                 // BMC watch dog timer reset output,       (* chip_pin = "E6"  *)   falling edge pulse, not used 
                                          
, input        BMC_MAIN_RESET_N            // BMC request to reset the system         (* chip_pin = "F5"  *)   falling edge pulse, reset BMC only                                                                                                                                                                                                                  
, input	       BMC_CPLD_RESET1             // spare reset signal from BMC,            (* chip_pin = "J3"  *)   falling edge pulse, reset Tomahawk 
, input	       BMC_CPLD_RESET2             // spare reset signal from BMC,            (* chip_pin = "K2"  *)   falling edge pulse, reset uServer  
, input	       LS_BMC_CPLD_RESET3             // spare reset signal from BMC             (* chip_pin = "L3"  *)   falling edge pulse, reset tomahawk, microserver and everything in main power domain 
, input	       LS_BMC_CPLD_RESET4             // spare reset signal from BMC             (* chip_pin = "N1"  *)   falling edge pulse, reset everything including BMC and standby power components, and tomahawk and main power domain components 
                                                                                    
, input	       PUSHBUTTON_RST_N            // front panel push btn reset              (* chip_pin = "K16" *) 
, input	       CPLD_PUSH_BTN_RST_IN_N      // on board debug reset btn                (* chip_pin = "L15" *) 
, input        DEBUG_RST_BTN_N             // Debug Reset button in Front panel is pressed, DEBUG_RST_BTN_N will become low (* chip_pin = "A15"  *)  
 
// reset request signal pulse                                       
, output	   pb_reset_neg_pul            // push button reset pulse
, output	   pb_dbg_reset_neg_pul        // on board pus button for debug
, output       facebook_dbg_reset_neg_pul  // facebook debug reset input

, output       bmc_req_reset_neg_pul       // bmc request reset input 
                                        
, output       bmc_reset_th_neg_pul      // bmc reset request to cpld, input #1
, output       bmc_reset_usrv_neg_pul    // bmc reset request to cpld, input #2
, output       bmc_reset_main_neg_pul    // bmc reset request to cpld, input #3
, output       bmc_reset_all_neg_pul     // bmc reset request to cpld, input #4

, output       wdt_reset_1_neg_pul         // wdt_reset_1_neg_pul
, output       wdt_reset_2_neg_pul         // wdt_reset_2_neg_pul
                                           
, output       reg_hot_req_pul             // reg 0x31 bit 4 negative edge
, output       reg_warm_req_pul            // reg 0x31 bit 5 negative edge
, output       reg_cold_req_pul            // reg 0x31 bit 6 negative edge
, output       reg_pwr_req_pul             // reg 0x31 bit 7 negative edge

) ;
 
// reg 0x3a is BMC reset enable register
wire  bmc_req_reset_n    = (reg_bmc_rst_en[0]) ? BMC_MAIN_RESET_N : 1'b1;
wire  bmc_cpld_reset_1_n = (reg_bmc_rst_en[1]) ? BMC_CPLD_RESET1  : 1'b1;
wire  bmc_cpld_reset_2_n = (reg_bmc_rst_en[2]) ? BMC_CPLD_RESET2  : 1'b1;
wire  bmc_cpld_reset_3_n = (reg_bmc_rst_en[3]) ? LS_BMC_CPLD_RESET3  : 1'b1;
wire  bmc_cpld_reset_4_n = (reg_bmc_rst_en[4]) ? LS_BMC_CPLD_RESET4  : 1'b1;
wire  wdt_reset_1_n      = (reg_bmc_rst_en[5]) ? LS_BMC_WDTRST1      : 1'b1;
wire  wdt_reset_2_n      = (reg_bmc_rst_en[6]) ? LS_BMC_WDTRST2      : 1'b1;

// wire  bmc_reset_th       = BMC_CPLD_RESET1;
// wire  bmc_reset_usrv     = BMC_CPLD_RESET2;
// wire  bmc_reset_main     = LS_BMC_CPLD_RESET3;
// wire  bmc_reset_all      = LS_BMC_CPLD_RESET4;   

wire  pb_reset_n           = (reg_gen_rst_en[0]) ? PUSHBUTTON_RST_N       : 1'b1;
wire  pb_dbg_reset_n       = (reg_gen_rst_en[1]) ? CPLD_PUSH_BTN_RST_IN_N : 1'b1;
wire  facebook_dbg_reset_n = (reg_gen_rst_en[2]) ? DEBUG_RST_BTN_N        : 1'b1;

// push button reset

neg_edge	pb_reset_neg_edge        // push button reset, treated as cold reset
(
.clk	 		(clk),
.rst_n	 		(1'b1),
.sig_in     	(pb_reset_n),
.neg_edge_en	(pb_reset_neg_pul) 
) ;

// on board push button reset, reset microserver

neg_edge	pb_dbg_reset_neg_edge        // push button reset, treated as cold reset
(
.clk	 		(clk),
.rst_n	 		(1'b1),
.sig_in     	(pb_dbg_reset_n),
.neg_edge_en	(pb_dbg_reset_neg_pul) 
) ;
 
// on board push button reset, reset microserver

neg_edge	facebook_reset_neg_edge        // facebook debug header button reset, treated as cold reset
(
.clk	 		(clk),
.rst_n	 		(1'b1),
.sig_in     	(facebook_dbg_reset_n),
.neg_edge_en	(facebook_dbg_reset_neg_pul) 
) ;


// BMC main reset, BMC main reset, reset BMC only?? according to Tian

neg_edge	bmc_req_reset_neg_edge        //  
(
.clk	 		(clk),
.rst_n	 		(1'b1),
.sig_in     	(bmc_req_reset_n),
.neg_edge_en	(bmc_req_reset_neg_pul) 
) ;
 
// BMC main reset to cpld 1,  

neg_edge	bmc_cpld_reset_1_neg_edge        // bmc to cpld reset 1
(
.clk	 		(clk),
.rst_n	 		(1'b1),
.sig_in     	(bmc_cpld_reset_1_n),                                                    
.neg_edge_en	(bmc_reset_th_neg_pul)                                             
) ;                                                                                    
                                                                                        
// BMC main reset to cpld 2,  

neg_edge	bmc_cpld_reset_2_neg_edge        // bmc to cpld reset 2
(
.clk	 		(clk),
.rst_n	 		(1'b1),
.sig_in     	(bmc_cpld_reset_2_n),
.neg_edge_en	(bmc_reset_usrv_neg_pul) 
);

// BMC main reset to cpld 3  

neg_edge	bmc_cpld_reset_3_neg_edge        // bmc to cpld reset 3
(
.clk	 		(clk),
.rst_n	 		(1'b1),
.sig_in     	(bmc_cpld_reset_3_n),
.neg_edge_en	(bmc_reset_main_neg_pul) 
);

// BMC main reset to cpld 4,  

neg_edge	bmc_cpld_reset_4_neg_edge        // bmc to cpld reset 4
(
.clk	 		(clk),
.rst_n	 		(1'b1),
.sig_in     	(bmc_cpld_reset_4_n),
.neg_edge_en	(bmc_reset_all_neg_pul) 
) ; 

// watch dog timeout 1

neg_edge	wdt_1_neg_edge        // watchdog timer 1
(
.clk	 		(clk),
.rst_n	 		(1'b1),
.sig_in     	(wdt_reset_1_n),
.neg_edge_en	(wdt_reset_1_neg_pul) 
) ; 
 

// watch dog timeout 2

neg_edge	wdt_2_neg_edge        // watchdog timer 1
(
.clk	 		(clk),
.rst_n	 		(1'b1),
.sig_in     	(wdt_reset_2_n),
.neg_edge_en	(wdt_reset_2_neg_pul) 
) ; 
 

// SW reset pulse, positive edge
wire  reg_pwr_req  =  (reg_rst_ctrl_wr & !reg_rst_ctrl[7] );
pos_edge	sw_pwr_pos_edge        
(
.clk	 		(clk),
.rst_n	 		(1'b1),
.sig_in     	(reg_pwr_req),
.pos_edge_en	(reg_pwr_req_pul) 
);

// SW reset pulse, positive edge
wire  reg_cold_req  =  (reg_rst_ctrl_wr & !reg_rst_ctrl[6] );
pos_edge	sw_cold_pos_edge        
(
.clk	 		(clk),
.rst_n	 		(1'b1),
.sig_in     	(reg_cold_req),
.pos_edge_en	(reg_cold_req_pul) 
);

// SW reset pulse, positive edge
wire  reg_warm_req  =  (reg_rst_ctrl_wr & !reg_rst_ctrl[5] );
pos_edge	sw_warm_pos_edge        
(
.clk	 		(clk),
.rst_n	 		(1'b1),
.sig_in     	(reg_warm_req),
.pos_edge_en	(reg_warm_req_pul) 
);

// SW reset pulse, positive edge
wire  reg_hot_req  =  (reg_rst_ctrl_wr & !reg_rst_ctrl[4] );
pos_edge	sw_hot_pos_edge        
(
.clk	 		(clk),
.rst_n	 		(1'b1),
.sig_in     	(reg_hot_req),
.pos_edge_en	(reg_hot_req_pul) 
);

 
 
 

endmodule
