 /*
facebook wedge100 
Model:  system_boot module
Revision: V0.1, 
History : 
2015/02/20 - initial draft by zhiping Yao

reset reason
1: Power on reset, both standby and main power domain
2: poweron reset, only tomahawk and microserver power cycling, BMC keep working
3: cold reset: reset both BMC and tomahawk/microserver
4: warm reset: only reset micro-server and tomahawk


*/


//  control register bit definition
//  
// reg 0x31 reset control register bit definition
// bit7 Power_Reset    : this bit will drive power607 to shutdown power. this is real power cycling on board. active low
// bit6 reset_all_warm : write 0 to this bit will cause warm boot for all devices including cpu itself
// bit5 reset_all_cold : write 0 to this bit will cause cold boot for all devices including cpu itself
// bit4 reset_cpu      : write 0 to this bit will cause warm boot for cpu system only including NOR flash, but not MAC and other devices
// bit3 reserved       : reserved
// bit2 mac_reset      : 0 will reset MAC
// bit1 reserved
// bit0 reserved

//-----------------------------------------------------------------------------
// Title       : system boot module
// Design      : Wedge100 CPLD Design
// File        : system_boot.v
//-----------------------------------------------------------------------------
// Version     : Ver 2.0
// Changes     : 1. Renamed the signals mentioned in excel sheet
//-----------------------------------------------------------------------------
// Description:- System boot implementation
//-----------------------------------------------------------------------------

module system_boot #
(
    //spatha/sic use the following parameter, helix4 need minimal
    parameter [27:00]   PULSE_WIDTH_HOT    = 5000000 , // 100ms - clock= 50MHz
    parameter [27:00]   PULSE_WIDTH_WARM   = 10000000, // 200ms - clock= 50MHz
    parameter [27:00]   PULSE_WIDTH_COLD   = 19000000, // 380ms - clock= 50MHz
    parameter [27:00]   PULSE_WID_BMC_MAIN = 1000000 , // 20ms  - clock= 50MHz
    parameter [27:00]   PULSE_WID_PUSH_BTN = 1000000 , // 20ms  - clock= 50MHz
    parameter           SW_DEB_LEN         = 32
)
(
	input		 clk	          // clock
,	input		 clk_1s			  // clock 1second for watchdog timer
,   input        clk_100ms        // clk_100hz
,   input        reg_rst_ctrl_wr  
                                  
,   input [7:0]  rw_0x30          // reg 0x30
,   input [7:0]  rw_0x31          // reg 0x31
,   input [7:0]  rw_0x32          // reg 0x32
,   input [7:0]  rw_0x33          // reg 0x33
,   input [7:0]  rw_0x34          // reg 0x34
,   input [7:0]  rw_0x35          // reg 0x35
,   input [7:0]  rw_0x36          // reg 0x36
,   input [7:0]  rw_0x37          // reg 0x37
,   input [7:0]  rw_0x38          // reg 0x38
,   input [7:0]  rw_0x39          // reg 0x39
,   input [7:0]  rw_0x3a          // reg 0x3a
,   input [7:0]  rw_0x3b          // reg 0x3b

,   output [7:0] ro_0x0c          //
,   output [7:0] ro_0x0d          //
,   output [7:0] ro_0x0e          //  
,   output [7:0] ro_0x0f          //  

// PWR1014A reset output 
,   input        pd_stby_reset_n      // async standby power on reset, active low, from power1014a, already delayed
,   input	     pd_main_reset_n       // power domain main  
,   input        rst_pwr_n             // self poweron reset, only used for reset generation  

// BMC control input signals
,   input        xp_powerup            // tomahawk power up, main power up except userver, driven by BMC
,   input        userver_powerup       // microserver power up , driven by BMC                                                   
                        
// reset trigger                                     
,   input        MICROSRV_PCIE_RST_N         // micro-server reset output               (* chip_pin = "E7"  *)  : Microserver request to reset PCIe devices                                                                                                   
,   input	     LS_BMC_WDTRST1                 // BMC watch dog timer reset output,       (* chip_pin = "B5"  *)   falling edge pulse 
,   input	     LS_BMC_WDTRST2                 // BMC watch dog timer reset output,       (* chip_pin = "E6"  *)   falling edge pulse 
                                          
,   input        BMC_MAIN_RESET_N            // BMC request to reset the system         (* chip_pin = "F5"  *)  :                                                                                                                                                                                                                  
,   input	     BMC_CPLD_RESET1             // spare reset signal from BMC,            (* chip_pin = "J3"  *)    falling edge pulse, reset BMC only 
,   input	     BMC_CPLD_RESET2             // spare reset signal from BMC,            (* chip_pin = "K2"  *)    falling edge pulse, reset uServer  
,   input	     LS_BMC_CPLD_RESET3             // spare reset signal from BMC             (* chip_pin = "L3"  *)
,   input	     LS_BMC_CPLD_RESET4             // spare reset signal from BMC             (* chip_pin = "N1"  *)
                                                                                    
,   input	     PUSHBUTTON_RST_N            // front panel push btn reset              (* chip_pin = "K16" *) 
,   input	     CPLD_PUSH_BTN_RST_IN_N      // on board debug reset btn                (* chip_pin = "L15" *) 
,   input        DEBUG_RST_BTN_N             // Debug Reset button in Front panel is pressed, DEBUG_RST_BTN_N will become low (* chip_pin = "A15"  *)  
,   input        pwr_usrv_button_in          // micro-server power button
 
//output signals
// power control register 0x30             
,	output	     pwr_cyc_all_n         // power cycle output to pwr1014a, power cycle all power
,   output       pwr_main_en           // enable main power, active high, to pwr1014a
,   output       pwr_usrv_en           // enable micro-server power, active high, to pwr1014a
,   output       pwr_usrv_btn_n        // micro-server power button input
                 
,   output       pwr_main_force        // force main power, debug only, should not change by SW
,   output       pwr_mserver_force     // force micro-server power, debug only, should not change by SW
                                     
// reset control reset register 0x32   	 
,   output       microsrv_rst_n        // micro-server reset output 
,   output       iso_xpliant_rst_n     // tomahawk reset output 
// ,   output       tomahawk_pcie_rst_n   // tomahawk pcie reset output 

// ,   output       dpll_clk_sel          // dpll clock select// BMC reset
,   output       comexp_test_en_n        // comexp test card enable
,   output       usrv_uart_iso_en      // micro-server uart isolation buffer 0: disable, 1: enable, under BMC control

,   output       bmc_rst_n            // bmc reset output 
   
// qsfp reset output , reg 0x34 to 0x37                                                                                                      
,   output [31:0] qsfp_reset_n         
                                  
// reset control register 0x32 for small ICs, such as oob switch, phy, and fan-ctrl card    
,   output       fancard_rst_n         // fan_ctrl card reset output                               
,	output	     usb_brdg_rst_n        // usb bridge reset output 
,   output       usb_hub_rst_n         // usb hub reset output 
,   output       bcm5387_rst_n         // OOB switch BCM5387 reset output  
,   output       fp_phy_rst_n          // front port PHY reset output 
,   output       bcm54616_phy_rst_n    // on-board PHY BCM54616 reset output 
,   output       usb_spi_brg_rst_n     // reserved usb to spi bridge reset output
,   output       dpll_rst_n            // reserved for dpll module reset

// reset control register 0x33  for i2c mux reset                                
,	output	     i2c_mux_rst0_n        // i2c mux 0 reset output
,   output       i2c_mux_rst1_n        // i2c mux 1 reset output
,   output       i2c_mux_rst2_n        // i2c mux 2 reset output
,   output       i2c_mux_rst3_n        // i2c mux 3 reset output
,   output       i2c_mux_psu_rst_n     // i2c mux for psu reset output 
,   output       i2c_mux_exp_rst_n     // i2c mux for qsfp LLI signals reset output  
                 
,   output		 hot_reset_n 	       // cold reset output, reset all devices on board, both standby and main power domain
,   output       warm_reset_n          // warm reset output, reset standby power domain                          	 
,   output       cold_reset_n          // hot reset output, reset BMC only, will not affect datapath             

) ;
 
wire       pwr_usrv_btn_n_out; 
reg  [1:0] pwr_usrv_button_in_seq;

// filtering button signal
// wire  pwr_usrv_button_in = BMC_READY_N ? BMC_PWR_BTN_OUT_N : BMC_PWR_BTN_IN_N;
always @ (posedge clk_100ms )
 begin                                                    
  pwr_usrv_button_in_seq[0] =  pwr_usrv_button_in          ;                      
  pwr_usrv_button_in_seq[1] =  pwr_usrv_button_in_seq[0]   ;              
 end

 // usrv power button output 
assign    pwr_usrv_btn_n = pwr_usrv_button_in_seq[1] & pwr_usrv_btn_n_out ; 


// ************************************************************************
//  boot reset pulse generation
wire    pb_reset_neg_pul           ;
wire    pb_dbg_reset_neg_pul       ;
wire    facebook_dbg_reset_neg_pul ;
wire    bmc_req_reset_neg_pul      ;
        
wire    bmc_reset_th_neg_pul       ;
wire    bmc_reset_usrv_neg_pul     ;
wire    bmc_reset_main_neg_pul     ;
wire    bmc_reset_all_neg_pul      ;
wire    wdt_reset_1_neg_pul        ;
wire    wdt_reset_2_neg_pul        ;
        
wire    reg_hot_req_pul            ;
wire    reg_warm_req_pul           ;
wire    reg_cold_req_pul           ;
wire    reg_pwr_req_pul            ;
 
wire  hot_reset_n_int  ;
wire  warm_reset_n_int ;
wire  cold_reset_n_int ;
wire 	xplaint_reset_n_int;
wire  reset_usrv_n     ;
wire  reset_th_n       ;
 
assign  hot_reset_n  = hot_reset_n_int  ;
assign  warm_reset_n = warm_reset_n_int ;
assign  cold_reset_n = cold_reset_n_int ;

wire  [2:0] reg_gen_rst_en;

assign   reg_gen_rst_en = rw_0x3b[2:0] ;

wire  microsrv_pcie_rst_n_int = MICROSRV_PCIE_RST_N  & rw_0x3b[3] ; 

//  BMC GPIO power up tomahawk enable
wire  th_powerup_int          = (rw_0x3b[4]) ? xp_powerup      : 1'b1 ;   // in default tomahawk power is enabled  by rw_0x3b[4]
wire  userver_powerup_int     = (rw_0x3b[5]) ? userver_powerup : 1'b1 ;   // in default microserver power is ebaled by rw_0x3b[5]

 
reset_trigger reset_trigger_u 
(
	.clk					        (clk					    ),  // clock 
 
	.reg_rst_ctrl_wr                (reg_rst_ctrl_wr            ),  // write access to reg 0x31, used to generate pulse, active high                
	.reg_rst_ctrl                   (rw_0x31                    ),  // reset control register, 0x31 
	.reg_bmc_rst_en                 (rw_0x3a                    ),  // bmc reset enable control register
	.reg_gen_rst_en                 (reg_gen_rst_en             ),  // general reset enable control register
  
	//.MICROSRV_PCIE_RST_N            (MICROSRV_PCIE_RST_N        ),  // micro-server reset output               (* chip_pin = "E7"  *)  : Microserver request to reset PCIe devices   
                                                         
	.LS_BMC_WDTRST1                    (LS_BMC_WDTRST1                ),  // BMC watch dog timer reset output,       (* chip_pin = "B5"  *)   falling edge pulse 
	.LS_BMC_WDTRST2                    (LS_BMC_WDTRST2                ),  // BMC watch dog timer reset output,       (* chip_pin = "E6"  *)   falling edge pulse 
	 
	.BMC_MAIN_RESET_N               (BMC_MAIN_RESET_N           ),  // BMC request to reset the system         (* chip_pin = "F5"  *)  :                                                                                                                                                                                                                  
	.BMC_CPLD_RESET1                (BMC_CPLD_RESET1            ),  // spare reset signal from BMC,            (* chip_pin = "J3"  *)    falling edge pulse, reset BMC only 
	.BMC_CPLD_RESET2                (BMC_CPLD_RESET2            ),  // spare reset signal from BMC,            (* chip_pin = "K2"  *)    falling edge pulse, reset uServer  
	.LS_BMC_CPLD_RESET3                (LS_BMC_CPLD_RESET3            ),  // spare reset signal from BMC             (* chip_pin = "L3"  *)
	.LS_BMC_CPLD_RESET4                (LS_BMC_CPLD_RESET4            ),  // spare reset signal from BMC             (* chip_pin = "N1"  *)
                                          
	.PUSHBUTTON_RST_N               (PUSHBUTTON_RST_N           ),  // front panel push btn reset              (* chip_pin = "K16" *) 
	.CPLD_PUSH_BTN_RST_IN_N         (CPLD_PUSH_BTN_RST_IN_N     ),  // on board debug reset btn                (* chip_pin = "L15" *) 
	.DEBUG_RST_BTN_N                (DEBUG_RST_BTN_N            ),  // Debug Reset button in Front panel is pressed, DEBUG_RST_BTN_N will become low (* chip_pin = "A15"  *)  
                                    
    // request signal pulse              
	.pb_reset_neg_pul           	(pb_reset_neg_pul           ),  // push button reset pulse
	.pb_dbg_reset_neg_pul       	(pb_dbg_reset_neg_pul       ),  // on board pus button for debug
	.facebook_dbg_reset_neg_pul     (facebook_dbg_reset_neg_pul ),  // facebook debug reset input
	
	.bmc_req_reset_neg_pul          (bmc_req_reset_neg_pul      ),  // bmc request reset input 
	 
	.bmc_reset_th_neg_pul           (bmc_reset_th_neg_pul       ),  // bmc reset request to cpld, input #1
	.bmc_reset_usrv_neg_pul         (bmc_reset_usrv_neg_pul     ),  // bmc reset request to cpld, input #2
	.bmc_reset_main_neg_pul         (bmc_reset_main_neg_pul     ),  // bmc reset request to cpld, input #3
	.bmc_reset_all_neg_pul          (bmc_reset_all_neg_pul      ),  // bmc reset request to cpld, input #4
	
	.wdt_reset_1_neg_pul            (wdt_reset_1_neg_pul        ),  // wdt_reset_1_neg_pul
	.wdt_reset_2_neg_pul            (wdt_reset_2_neg_pul        ),  // wdt_reset_2_neg_pul
	   
	.reg_hot_req_pul                (reg_hot_req_pul            ),  // reg 0x30 bit 4 negative edge
	.reg_warm_req_pul               (reg_warm_req_pul           ),  // reg 0x30 bit 5 negative edge
	.reg_cold_req_pul               (reg_cold_req_pul           ),  // reg 0x30 bit 6 negative edge
	.reg_pwr_req_pul                (reg_pwr_req_pul            )   // reg 0x30 bit 7 negative edge

) ;
 
 

//===========================================================
 

 reset_ctrl reset_ctrl_u
(
	.clk				           (clk			              ),     // clock 
	.pd_stby_reset_n 	           (pd_stby_reset_n 	      ),     // async standby power on reset, active low, from power1014a, already delayed
	.pd_main_reset_n               (pd_main_reset_n           ),     // power domain main
	//.main_stby_reset_n             (main_stby_reset_n         ),   // async main power on reset, active low, from power1014a, already delayed
	.rst_pwr_n                     (rst_pwr_n                 ),     // self poweron reset, only used for reset generation 
   .cb_reset							 (microsrv_pcie_rst_n_int),
	.pb_reset_neg_pul              (pb_reset_neg_pul          ),     // push button reset pulse            
	.pb_dbg_reset_neg_pul          (pb_dbg_reset_neg_pul      ),     // on board pus button for debug      
	.facebook_dbg_reset_neg_pul    (facebook_dbg_reset_neg_pul),     // facebook debug reset input         
	.bmc_req_reset_neg_pul         (bmc_req_reset_neg_pul     ),     // bmc request reset input            
	                                  
	.bmc_reset_th_neg_pul          (bmc_reset_th_neg_pul      ),     // bmc reset request to cpld, input #1
	.bmc_reset_usrv_neg_pul        (bmc_reset_usrv_neg_pul    ),     // bmc reset request to cpld, input #2
	.bmc_reset_main_neg_pul        (bmc_reset_main_neg_pul    ),     // bmc reset request to cpld, input #3
	.bmc_reset_all_neg_pul         (bmc_reset_all_neg_pul     ),     // bmc reset request to cpld, input #4
	
	.wdt_reset_1_neg_pul           (wdt_reset_1_neg_pul       ),     // wdt_reset_1_neg_pul                
	.wdt_reset_2_neg_pul           (wdt_reset_2_neg_pul       ),     // wdt_reset_2_neg_pul                
                                  
	.reg_hot_req_pul               (reg_hot_req_pul           ),     // reg 0x30 bit 4 negative edge       
	.reg_warm_req_pul              (reg_warm_req_pul          ),     // reg 0x30 bit 5 negative edge       
	.reg_cold_req_pul              (reg_cold_req_pul          ),     // reg 0x30 bit 6 negative edge       
	.reg_pwr_req_pul               (reg_pwr_req_pul           ),     // reg 0x30 bit 7 negative edge       
         
	.hot_reset_n 	               (hot_reset_n_int 	      ),     // cold reset output, reset all devices on board, both standby and main power domain
	.warm_reset_n                  (warm_reset_n_int          ),     // warm reset output, reset standby power domain
	.cold_reset_n                  (cold_reset_n_int          ),     // hot reset output, reset BMC only, will not affect datapath
	.reset_usrv_n                  (reset_usrv_n              ),     // micro server reset request 
	.reset_th_n                    (reset_th_n                ),      // tomahawk reset request     
	.xplaint_reset						 (xplaint_reset_n_int)
) ;

//========================================================================================
wire   bmc_rst_n_int;
assign bmc_rst_n = bmc_rst_n_int & pd_stby_reset_n & PUSHBUTTON_RST_N;

reset_out reset_out_u
(
	.clk                  (clk                     ),
	.hot_reset_n          (hot_reset_n_int         ),     // hot reset, from reset_ctrl module 
	.warm_reset_n         (warm_reset_n_int        ),     // warm reset, from reset_ctrl module 
	.cold_reset_n         (cold_reset_n_int        ),     // cold reset, from reset_ctrl module
	.reset_usrv_n         (reset_usrv_n            ),     // micro server reset request 
	.reset_th_n           (reset_th_n              ),     // tomahawk reset request     
   .xplaint_reset_n		 (xplaint_reset_n_int),                                                 
// BMC control input signals                        
	.xp_powerup           (th_powerup_int          ),     // tomahawk power up, main power up except userver
	.userver_powerup      (userver_powerup_int     ),     // microserver power up  
	.microsrv_pcie_rst_n  (microsrv_pcie_rst_n_int ),   // microserver pcie reset input
	                                           
	.rw_0x30              (rw_0x30                 ),     // reg 0x30
//	.rw_0x31              (rw_0x31                 ),     // reg 0x31
	.rw_0x32              (rw_0x32                 ),     // reg 0x32
	.rw_0x33              (rw_0x33                 ),     // reg 0x33
	.rw_0x34              (rw_0x34                 ),     // reg 0x34
	.rw_0x35              (rw_0x35                 ),     // reg 0x35
	.rw_0x36              (rw_0x36                 ),     // reg 0x36
	.rw_0x37              (rw_0x37                 ),     // reg 0x37
	.rw_0x38              (rw_0x38                 ),     // reg 0x38
	.rw_0x39              (rw_0x39                 ),     // reg 0x39
                                                   
                                                   
// power control register 0x30                     
	.pwr_cyc_all_n    	  (pwr_cyc_all_n           ),     // power cycle output to pwr1014a, power cycle all power
	.pwr_main_en          (pwr_main_en             ),     // enable main power, active high, to pwr1014a
	.pwr_usrv_en          (pwr_usrv_en             ),     // enable micro-server power, active high, to pwr1014a
	.pwr_usrv_btn_n       (pwr_usrv_btn_n_out      ),     // micro-server power button
                                                   
	.pwr_main_force       (pwr_main_force          ),     // force main power, debug only, should not change by SW
	.pwr_mserver_force    (pwr_mserver_force       ),     // force micro-server power, debug only, should not change by SW
                                               
// reset control reset register 0x32   	       
	.microsrv_rst_n       (microsrv_rst_n          ),   // micro-server reset output 
	.iso_xpliant_rst_n    (iso_xpliant_rst_n      ),   // tomahawk reset output 
	// .tomahawk_pcie_rst_n  (tomahawk_pcie_rst_n     ),   // tomahawk pcie reset output 
                                                   
	// .dpll_clk_sel         (dpll_clk_sel            ),     // dpll clock select  
	.comexp_test_en_n     (comexp_test_en_n        ),    // comexp test card enable     
	.usrv_uart_iso_en     (usrv_uart_iso_en        ),    // micro-server uart isolation buffer 0: disable, 1: enable, under BMC control                                        
// BMC reset                                       
	.bmc_rst_n            (bmc_rst_n_int           ),  // bmc reset output 
                                                   
// qsfp reset output , reg 0x34 to 0x37                                                                                                          
	.qsfp_reset_n         (qsfp_reset_n            ),   
                                                   
// reset control register 0x32 for small ICs,      such as oob switch, phy, and fan-ctrl card    
	.fancard_rst_n        (fancard_rst_n           ),   // fan_ctrl card reset output                               
	.usb_brdg_rst_n       (usb_brdg_rst_n          ),   // usb bridge reset output 
	.usb_hub_rst_n        (usb_hub_rst_n           ),   // usb hub reset output 
	.bcm5387_rst_n        (bcm5387_rst_n           ),   // OOB switch BCM5387 reset output  
	.fp_phy_rst_n         (fp_phy_rst_n            ),   // front port PHY reset output 
	.bcm54616_phy_rst_n   (bcm54616_phy_rst_n      ),   // on-board PHY BCM54616 reset output 
	.usb_spi_brg_rst_n    (usb_spi_brg_rst_n       ),    // reserved usb to spi bridge reset output
    .dpll_rst_n           (dpll_rst_n              ),   // reserved for dppl reset                                           
// reset control register 0x33  for i2c mux reset                                
	.i2c_mux_rst0_n       (i2c_mux_rst0_n          ),    // i2c mux 0 reset output
	.i2c_mux_rst1_n       (i2c_mux_rst1_n          ),    // i2c mux 1 reset output
	.i2c_mux_rst2_n       (i2c_mux_rst2_n          ),    // i2c mux 2 reset output
	.i2c_mux_rst3_n       (i2c_mux_rst3_n          ),    // i2c mux 3 reset output
	.i2c_mux_psu_rst_n    (i2c_mux_psu_rst_n       ),    // i2c mux for psu reset output 
	.i2c_mux_exp_rst_n    (i2c_mux_exp_rst_n       )     // i2c mux for qsfp LLI signals reset output  	                              
                   
) ;



//============================================================
reset_reason reset_reason_u

( 	
    .clk 			              (clk            		      ),      //  clock
    .reset_hot_n                  (hot_reset_n_int            ),      // hot reset output
    .reset_warm_n                 (warm_reset_n_int           ),      // warm reset output
    .reset_cold_n                 (cold_reset_n_int           ),      // cold reset output
                                                              
    .pd_stby_reset_n              (pd_stby_reset_n            ),      // async standby power on reset, active low, from power1014a, already delayed
    .pd_main_reset_n              (pd_main_reset_n            ),      // async main power on reset, active low, from power1014a, already delayed
                                                              
    .pb_reset_neg_pul             (pb_reset_neg_pul           ),      // push button reset pulse                         
    .pb_dbg_reset_neg_pul         (pb_dbg_reset_neg_pul       ),      // on board pus button for debug                   
    .facebook_dbg_reset_neg_pul   (facebook_dbg_reset_neg_pul ),      // facebook debug reset input                      
                                                           
    .bmc_req_reset_neg_pul        (bmc_req_reset_neg_pul      ),      // bmc request reset input                         
    .bmc_reset_th_neg_pul         (bmc_reset_th_neg_pul       ),      // bmc reset request to cpld, input #1             
    .bmc_reset_usrv_neg_pul       (bmc_reset_usrv_neg_pul     ),      // bmc reset request to cpld, input #2             
    .bmc_reset_main_neg_pul       (bmc_reset_main_neg_pul     ),      // bmc reset request to cpld, input #3             
    .bmc_reset_all_neg_pul        (bmc_reset_all_neg_pul      ),      // bmc reset request to cpld, input #4             
                                                                                                                        
    .wdt_reset_1_neg_pul          (wdt_reset_1_neg_pul        ),      // wdt_reset_1_neg_pul                             
    .wdt_reset_2_neg_pul          (wdt_reset_2_neg_pul        ),      // wdt_reset_2_neg_pul                             
                                                                               
    .reg_hot_req_pul              (reg_hot_req_pul            ),      // reg 0x30 bit 4 negative edge                    
    .reg_warm_req_pul             (reg_warm_req_pul           ),      // reg 0x30 bit 5 negative edge                    
    .reg_cold_req_pul             (reg_cold_req_pul           ),      // reg 0x30 bit 6 negative edge                    
    .reg_pwr_req_pul              (reg_pwr_req_pul            ),      // reg 0x30 bit 7 negative edge                    
            
 	.ro_0x0d                      (ro_0x0d                    ),      // reset reason register
 	.ro_0x0e                      (ro_0x0e                    ),      //  
    .ro_0x0f                      (ro_0x0f                    )       // reset reason status bit register
 
) ;


 

endmodule
                                